import java.io.*;
import java.util.Date;
import java.awt.*;
import java.util.Vector;

class OrderList extends List {

	public OrderList ()
	{
		super (10, false);
		setFont (new Font ("Courier", Font.PLAIN, 12));
	}

	public Dimension preferredSize ()
	{
		return new Dimension (275, 150);
	}
}

class DocketDisplay extends Dialog {

	Order theOrder;
	TextArea theText;
	CustDetails myCustDetails;
	Server theServer;
	Select theParent;
	Customer myCust;

	public DocketDisplay (CustDetails c, Order o, Server s, Customer cm, Select sel)
	{
		super ((Frame) c, "Docket Display", true);
		myCustDetails = c;
		myCust = cm;
		theOrder = o;
		theServer = s;
		theParent = sel;
		initScreen ();
		getText();
		show ();
	}

	public void initScreen ()
	{
		add ("Center", theText = new TextArea (30,45));
		theText.setFont (new Font ("Courier", Font.PLAIN, 12));

		Button b1, b2, b3, b4;

		Panel p = new Panel();
		p.add (b1 = new Button ("Same Again"));
			b1.setBackground (new Color (143, 200, 162));
		p.add (b2 = new Button ("Reprint"));
			b2.setBackground (new Color (145,196,255));
		p.add (b3 = new Button ("Edit"));
			b3.setBackground (new Color (145,196,255));
		p.add (b4 = new Button ("Cancel"));
			b4.setBackground (new Color (255,128,128));

		add ("South", p);

		pack ();
	}

	public void getText ()
	{
		try {
			String filename = "dockets/docket" + theOrder.orderNum;
			long filelen = new File(filename).length();
			//System.out.println ("filelen = " + filelen);
			DataInputStream fp = new DataInputStream (new FileInputStream (filename));
			byte Buffer[] = new byte [(int)filelen];
			fp.readFully (Buffer);
			fp.close ();
			theText.setText (new String (Buffer,0));
		}
		catch (IOException e)
		{
			e.printStackTrace ();
		}
	}

	public boolean handleEvent (Event evt)
	{
		if ((evt.arg == "Cancel") || (evt.id == Event.KEY_PRESS && evt.key == 0x1b))
		{
			dispose ();
		}
		else if ((evt.arg == "Edit") || (evt.id == Event.KEY_PRESS && evt.key == 'e'))
		{
			myCustDetails.dispose ();
			dispose ();
			new OrderEntry (theParent, myCust, theServer, theOrder);
		}
		else if ((evt.arg == "Reprint") || (evt.id == Event.KEY_PRESS && evt.key == 'r'))
		{
			dispose ();
			theOrder.printDocket (myCust);
		}
		else if ((evt.arg == "Same Again") || (evt.id == Event.KEY_PRESS && evt.key == 's'))
		{
			System.out.println ("Same again");
			myCustDetails.dispose ();
			dispose ();
			new OrderEntry (theParent, myCust, theServer, theOrder.orderNum);
		}

		return super.handleEvent (evt);
	}
}

class CustDetails extends GridFrame implements suburbTracker {


	InputField name, address, phone, minOrder;
	Label lastOrderNum, totalSpent, lastOrderDate;
	InputField discount;
	SuburbField suburb;
	InputField ubd;
	CommentArea comments;
	Button acceptBtn;
	Button lastOrderBtn;
	Button cancelBtn;
	Button badCustBtn;
	Button reportBtn;
	Server theServer;
	boolean newCust;
	Customer oldCust;
	Customer myCust;
	Label postcode;
	boolean newSubFlag;
	String orgComments;
	Select theParent;
	String oldName;
	boolean badCustFlag;
	Color badCustBG;
	Color goodCustBG;
	Color badCustInput;
	Color goodCustInput;
	OrderList orderList;
	Vector custOrders;

	public CustDetails (Select ss, Server s, String p, String n)
	{
		super ("New Customer");

		badCustBG = new Color (211,137,127);
		goodCustBG = new Color (207,207,183);
		badCustInput = new Color (231,160,150);
		goodCustInput = Color.white;

		theParent = ss;
		newCust = true;
		oldCust = null;
		oldName = null;
		theServer = s;
		myCust = new Customer();
		myCust.name = n;
		myCust.phone = p;
		myCust.lastOrderDate = new Date();
		badCustFlag = false;
		newSubFlag = false;
		initScreen ();
	}

	public CustDetails (Select ss, Server s, Customer oc)
	{
		super ("Modify Customer Details");

		badCustBG = new Color (211,137,127);
		goodCustBG = new Color (207,207,183);
		badCustInput = new Color (255,180,180);
		goodCustInput = Color.white;

		theParent = ss;
		newCust = false;
		oldCust = oc;
		theServer = s;
		badCustFlag = oc.badCust;
		if (badCustFlag)
			setBackground (badCustBG);
		myCust = oldCust;
		oldName = myCust.name;
		orgComments = myCust.comments;
		newSubFlag = false;
		initScreen ();
	}

	public void initScreen ()
	{
		Label l_name = new Label ("Name:", Label.RIGHT);
		name = new InputField (30, InputField.CAPS);

		Label l_discount = new Label ("% Disc:", Label.RIGHT);
		discount = new InputField (3, InputField.INT);

		Label l_address = new Label ("Address:", Label.RIGHT);
		address = new InputField (30, InputField.TEXT);

		Label l_phone = new Label ("Phone:", Label.RIGHT);
		phone = new InputField (20, InputField.TEXT);

		Label l_suburb = new Label ("Suburb:", Label.RIGHT);
		suburb = new SuburbField (theServer, this);

		postcode = new Label ("5000", Label.LEFT);

		Label l_ubd = new Label ("UBD:", Label.RIGHT);
		ubd = new InputField (5, InputField.CAPS);

		Label l_minOrder = new Label ("Min Order:", Label.RIGHT);
		minOrder = new InputField (6, InputField.MONEY);
		//minOrder.setProtected (true, 0);

		Label l_comments = new Label ("Comments:", Label.RIGHT);
		comments = new CommentArea (5, 40);

		Label l_orderList = new Label ("Orders", Label.CENTER);
		orderList = new OrderList ();

		Label l_lastOrderNum = new Label ("Last Order Num:", Label.RIGHT);
		lastOrderNum = new Label ("", Label.LEFT);
		lastOrderNum.setForeground (Color.blue);

		Label l_totalSpent = new Label ("Total Spent:", Label.RIGHT);
		totalSpent = new Label ("", Label.LEFT);
		totalSpent.setForeground (Color.blue);

		Label l_lastOrderDate = new Label ("Last Order Date:", Label.RIGHT);
		lastOrderDate = new Label ("", Label.LEFT);
		lastOrderDate.setForeground (Color.blue);

		if (newCust == false)
		{
			name.setText (oldCust.name);
			discount.setInt (oldCust.discount);
			address.setText (oldCust.address);
			phone.setText (oldCust.phone);
			suburb.setText (oldCust.suburb);
			ubd.setText (oldCust.ubd);
			setMinOrder ();
			comments.setText (oldCust.comments);
			lastOrderNum.setText ("" + oldCust.lastOrderNum);
			totalSpent.setText ("$ " + Money.toString(oldCust.totalSpent));
			lastOrderDate.setText (InputField.DateString(oldCust.lastOrderDate));

			// populate the order list
			custOrders = theServer.getOrdersByCust (oldCust);

			for (int i = custOrders.size()-1; i >= 0; i--)
			{
				Order myOrder = (Order) custOrders.elementAt (i);
				String s = "" + myOrder.orderNum + " " + InputField.DateString(myOrder.date) + "  $";
				String m = "" + Money.toString (myOrder.totalPrice());
				while (m.length() < 6) m = " " + m;
				orderList.addItem (s + m);
			}
		}
		else 
		{
			custOrders = null;
			name.setText (myCust.name);
			phone.setText (myCust.phone);
			lastOrderDate.setText (InputField.DateString(myCust.lastOrderDate));
			suburb.setText ("ADELAIDE");
			ubd.setText ("118");
			setMinOrder ();
		}

		acceptBtn = new Button ("New Order");
			acceptBtn.setBackground (new Color (143, 200, 162));

		badCustBtn = new Button ("Bad Cust");
			badCustBtn.setBackground (new Color (255,128,128));

		lastOrderBtn = new Button ("Change Last Order");
			lastOrderBtn.setBackground (new Color (145,196,255));

		reportBtn = new Button ("Customer Report");
			reportBtn.setBackground (new Color (145,196,255));

		cancelBtn = new Button ("Cancel");
			cancelBtn.setBackground (new Color (255,128,128));

		gridc.weightx = 0.0;
		addElement (EAST, l_name, 1, 1, 1, 1);
		addElement (WEST, name,   2, 1, 2, 1);
		addElement (EAST, l_address,   1, 2, 1, 1);
		addElement (WEST, address,   2, 2, 0, 1);
		addElement (EAST, l_phone,   1, 3, 1, 1);
		addElement (WEST, phone,   2, 3, 0, 1);
		addElement (EAST, l_suburb,   1, 4, 1, 1);
		addElement (WEST, suburb,   2, 4, 1, 1);
		addElement (WEST, postcode,   3, 4, 1, 1);
		addElement (EAST, l_ubd,   4, 4, 1, 1);
		addElement (WEST, ubd,   5, 4, 1, 1);
		addElement (EAST, l_minOrder,   6, 4, 1, 1);
		addElement (WEST, minOrder,   7, 4, 5, 1);
		gridc.weighty = 0.0;
		addElement (EAST, l_comments,   1, 5, 1, 2);
		addElement (WEST, comments,   2, 5, 3, 2);
		addElement (EAST, l_discount, 4, 1, 1, 1);
		addElement (WEST, discount, 5, 1, 1, 1);
		addElement (NORTH, l_orderList,   6, 5, 2, 1);
		gridc.weighty = 1.0;
		addElement (NORTHEAST, orderList,   5, 6, 3, 4);
		gridc.weighty = 0.0;
		addElement (EAST, l_lastOrderNum,   1, 7, 1, 1);
		addElement (WEST, lastOrderNum,   2, 7, 0, 1);
		addElement (EAST, l_totalSpent,   1, 8, 1, 1);
		addElement (WEST, totalSpent,   2, 8, 0, 1);
		addElement (EAST, l_lastOrderDate,   1, 9, 1, 1);
		addElement (WEST, lastOrderDate,   2, 9, 0, 1);
		gridc.weighty = 1.0;
		gridc.weightx = 0.0;

		Panel p1 = new Panel();
			p1.add (acceptBtn);
			p1.add (badCustBtn);
			p1.add (lastOrderBtn);
			p1.add (reportBtn);
			p1.add (cancelBtn);
		addElement (SOUTHEAST, p1, 3, 10, 5, 1);

		pack ();
		show ();
		name.requestFocus ();
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n')
		{
			if (evt.target == comments)
			{
				if (!evt.controlDown())
					return super.handleEvent (evt);
			}

			evt.id = Event.ACTION_EVENT;
			evt.target = acceptBtn;
			evt.arg = "New Order";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == '')
		{
			theServer.bumpTimer();
			evt.id = Event.ACTION_EVENT;
			evt.target = lastOrderBtn;
			evt.arg = "Change Last Order";
			super.deliverEvent (evt);
			return (true);
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Cancel"))
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.id == Event.LIST_SELECT && custOrders != null)
		{
			theServer.bumpTimer();
			int i = custOrders.size() - orderList.getSelectedIndex () - 1;
			Order myOrder = (Order) custOrders.elementAt (i);
			if (myOrder != null)
			{
				updateCustomer (); // could be last chance to save data before this 
									// popup terminates us
				new DocketDisplay (this, myOrder, theServer, myCust, theParent);
			}
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.arg == "New Order")
		{
			theServer.bumpTimer();
			boolean doIt = true;

			if (newCust == false)
			{
				String c = name.getText();
				if (c.length() > 0 && myCust.name.equals(c) == false &&
					 theServer.custExists (c))
				{
					new MessageBox (this, "Error", "This customer name has already been used");
					doIt = false;
				}
			}
				// for both new cust and old cust !
				if (address.getText().length() == 0)
				{
					address.requestFocus();
					doIt = false;
				}
				else if (phone.getText().length() == 0)
				{
					if (name.getText().length() == 0)
					{
						new MessageBox (this, "Error", "Must enter a NAME or a PHONE number");
						phone.requestFocus();
						doIt = false;
					}
				}

			if (doIt)
			{
				if (updateCustomer ())
				{
					dispose ();
					new OrderEntry (theParent, myCust, theServer);
				}
			}

			// bring up new order screen
		} else if (evt.arg == "Customer Report")
		{
			theServer.bumpTimer();
			theServer.printCustReport (myCust);
			new ReportDisplay (this, "Customer Report", "custReport");
		} else if (evt.arg == "Bad Cust")
		{
			theServer.bumpTimer();
			badCustFlag = !badCustFlag;
			Color iColor = goodCustInput;

			if (badCustFlag)
			{
				setBackground (badCustBG);
				iColor = badCustInput;
			}
			else
				setBackground (goodCustBG);

			myCust.badCust = badCustFlag;

			acceptBtn.setBackground (new Color (143, 200, 162));
			badCustBtn.setBackground (new Color (255,128,128));
			lastOrderBtn.setBackground (new Color (145,196,255));
			cancelBtn.setBackground (new Color (255,128,128));

			name.setBackground (iColor);
			discount.setBackground (iColor);
			address.setBackground (iColor);
			phone.setBackground (iColor);
			minOrder.setBackground (iColor);
			ubd.setBackground (iColor);
			comments.setBackground (iColor);
			suburb.setBackground (iColor);

			comments.requestFocus();

			updateCustomer ();
		} else if (evt.arg == "Change Last Order")
		{
			myCust.minOrder = minOrder.getText();

			// bring up last order - but this will do for now
			if (newCust == false)
			{

				Order theOrder = theServer.getOrder (myCust.lastOrderNum);
				if (theOrder != null)
				{
					dispose ();
					new OrderEntry (theParent, myCust, theServer, theOrder);
				}
			}
		}
		return (super.handleEvent (evt));
	}

	public boolean updateCustomer ()
	{
		boolean doIt = true;

		myCust.name = name.getText();
		myCust.address = address.getText();
		myCust.phone = phone.getText();
		myCust.suburb = suburb.getText();
		myCust.ubd = ubd.getText();
		myCust.comments = comments.getText();
		myCust.minOrder = "$ " + minOrder.getText();
		myCust.badCust = badCustFlag;
		myCust.discount = discount.getInt();

		theServer.setMinOrder (myCust.suburb, minOrder.getDouble());

		if (newCust)
		{
			doIt = theServer.newCust (myCust);
		}
		else
		{
			theServer.updateCust (myCust);
		}

		if (newSubFlag)
		{
			Suburb mySuburb = new Suburb ();
			mySuburb.suburb = suburb.getText();
			mySuburb.postcode = 0;
			mySuburb.minOrder = minOrder.getDouble();
			mySuburb.ubd = ubd.getText();
			theServer.newSuburb (mySuburb);
		}

		return doIt;
	}

	public void setMinOrder ()
	{
		Suburb mySuburb = theServer.getSuburb (suburb.getText());
		if (mySuburb != null)
		{
			postcode.setText ("" + mySuburb.postcode);
			minOrder.setDouble (mySuburb.minOrder);
			ubd.setText (mySuburb.ubd);
		}
		else
		{
			System.out.println ("Customer has an unknwon suburb !\n");
			suburb.setText (suburb.getText());
			postcode.setText ("5000");
			ubd.setText ("118");
			NewSuburb ();
		}
	}
	
	public void ClearMinOrder ()
	{
		//minOrder.setProtected (true, 0);
		newSubFlag = false;
	}
	
	public void NewSuburb ()
	{
		System.out.println ("Set newSubFlag = true !");
		minOrder.setProtected (false, InputField.MONEY);
		minOrder.setDouble (20.0);
		newSubFlag = true;
	}

	public void SuburbHasChanged (Suburb newSuburb)
	{
		postcode.setText ("" + newSuburb.postcode);
		minOrder.setDouble (newSuburb.minOrder);
		ubd.setText (newSuburb.ubd);
	}
}

class CustDetailsSelectList extends List {

	public CustDetailsSelectList ()
	{
		super (10, false);
	}

	public Dimension preferredSize ()
	{
		return new Dimension (400, 200);
	}
}

class CustDetailsSelect extends GridFrame {

	CustDetailsSelectList custList;
	Button okBtn;
	Button cancelBtn;
	Server theServer;
	Vector theResults;
	Select theParent;

	public CustDetailsSelect (Select ss, Server s, Vector r)
	{
		super ("Select Customer");
		theParent = ss;
		theServer = s;
		theResults = r;
		initScreen ();
	}

	public void initScreen ()
	{
		custList = new CustDetailsSelectList ();
		custList.setBackground (Color.white);
		fillCustList ();

		okBtn = new Button ("OK");
			okBtn.setBackground (new Color (145,200,162));

		cancelBtn = new Button ("Cancel");
			cancelBtn.setBackground (new Color (255,128,128));

		gridc.fill = 1;
		addElement (NORTHWEST, custList, 1,1,1,0);

		gridc.ipadx = gridc.ipady = 20;
		addElement (SOUTHEAST, okBtn,   2,2,1,1);
		addElement (SOUTHEAST, cancelBtn,2,3,1,1);

		pack();
		show();
		custList.requestFocus ();
	}

	void fillCustList ()
	{
		custList.clear ();
		for (int i =0; i < theResults.size(); i++)
		{
			Customer myCust = (Customer) theResults.elementAt (i);
			custList.addItem (myCust.name + " " + myCust.phone + " " + myCust.suburb);
		}
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n' && evt.target == custList)
		{
			evt.id = Event.ACTION_EVENT;
			evt.target = okBtn;
			evt.arg = "OK";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Cancel"))
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.arg == "OK")
		{
			int i = custList.getSelectedIndex ();
			if (i < 0) i = 0;
			Customer myCust = (Customer) theResults.elementAt (i);
			dispose ();
			new CustDetails (theParent, theServer, myCust);
		}

		return (super.handleEvent (evt));
	}
}
