import java.io.*;
import java.util.Date;
import java.awt.*;
import java.util.Vector;

class Customer extends Object {

	String name, address, phone, suburb, ubd, comments;
	long lastOrderNum;
	double totalSpent;
	Date lastOrderDate;
	String minOrder;
	long ID;
	boolean badCust;
	int discount;

	public Customer ()
	{
		name = null;
		address = null;
		phone = null;
		suburb = null;
		ubd = null;
		comments = null;
		lastOrderNum = 0;
		totalSpent = 0.0;
		lastOrderDate = null;
		ID = 0;
		badCust = false;
		discount = 0;
	}

	public Customer (DataInputStream fp, int Version) throws IOException
	{
		read (fp, Version);
	}

	public void read (DataInputStream in, int Version) throws IOException
	{
		if (Version >= 1)
		{
			ID = in.readLong ();
		}
		name = in.readUTF();
		address = in.readUTF();
		phone = in.readUTF();
		suburb = in.readUTF();
		ubd = in.readUTF();
		comments = in.readUTF();
		lastOrderNum = in.readLong();
		totalSpent = in.readDouble();
		lastOrderDate = new Date(in.readLong());

		if (Version >= 2)
		{
			badCust = in.readBoolean ();

			if (Version >= 3)
			{
				discount = in.readInt();
			}
		}
	}

	public void write (DataOutputStream out) throws IOException
	{
		out.writeLong(ID);
		out.writeUTF(name);
		out.writeUTF(address);
		out.writeUTF(phone);
		out.writeUTF(suburb);
		out.writeUTF(ubd);
		out.writeUTF(comments);
		out.writeLong(lastOrderNum);
		out.writeDouble(totalSpent);
		long theDate = 0;
		try {
			theDate = lastOrderDate.getTime();
		} catch (Exception e)
		{
		}

		out.writeLong(theDate);
		out.writeBoolean(badCust);
		out.writeInt(discount);
	}

	public void print (PrintStream out) throws IOException
	{
		String t = "\t";

		String newComment = comments;
		int i;
		while ((i = newComment.indexOf ('\n')) != -1)
			newComment = newComment.substring (0, i) + "\\n" + newComment.substring (i+1);
		while ((i = newComment.indexOf ('\t')) != -1)
			newComment = newComment.substring (0, i) + "\\t" + newComment.substring (i+1);
		System.out.println ("Converted '" + comments + "' into '" + newComment + "'");

		out.println ("" + ID + t + name + t + address + t + phone + t + suburb + t +
			t + ubd + t + newComment + t + lastOrderNum + t + totalSpent + t + discount);
	}

	public void printHeader (PrintStream out) throws IOException
	{
		out.println ("ID\tName\tAddress\tPhone\tSuburb\tUBD\tComments\tLastOrderNum\tTotalSpent\tDiscount");
	}

	public void importString (String s)
	{
		String t = "\t";
		int i;

		i = s.indexOf (t);
		ID = Integer.parseInt (s.substring (0, i)); s = s.substring (i+1); i = s.indexOf (t);
		name = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		address = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		phone = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		suburb = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		suburb = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		ubd = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);

		String newComments = s.substring (0,i);

		int x;
		while ((x = newComments.indexOf ("\\n")) != -1)
			newComments = newComments.substring (0, x) + '\n' + newComments.substring (x+2);
		while ((x = newComments.indexOf ("\\t")) != -1)
			newComments = newComments.substring (0, x) + '\n' + newComments.substring (x+2);
		comments = newComments; s = s.substring (i+1); i = s.indexOf (t);

		lastOrderNum = Integer.parseInt (s.substring (0, i)); s = s.substring (i+1); i = s.indexOf (t);
		totalSpent = new Double(s).doubleValue(); s = s.substring(i+1); i = s.indexOf (t);
		discount = Integer.parseInt (s.substring (0,i));
	}
}

class CommentArea extends TextArea {

	public CommentArea (int rows, int cols)
	{
		super (rows, cols);
	}

	public boolean handleEvent (Event e)
	{
		if (e.id == Event.KEY_PRESS)
		{
			switch (e.key)
			{
				case '\n':
				case '':
					// let parent handle this
					return false;
				case '\t':
					nextFocus();
					return true;
			}
		}
		return super.handleEvent (e);
	}
}

class CustEdit extends GridFrame {

	InputField name, address, phone, suburb, minOrder, lastOrderNum, totalSpent, lastOrderDate, discount;
	CommentArea comments;
	Button acceptBtn;
	Button cancelBtn;
	Button badCustBtn;
	Server theServer;
	boolean newCust;
	Customer oldCust;
	CustTable parentList;
	Customer myCust;
	String oldName;
	CreateMenu CM;
	boolean badCustFlag;
	Color badCustBG;
	Color goodCustBG;
	Color badCustInput;
	Color goodCustInput;

	public CustEdit (CreateMenu c, Server s, CustTable l)
	{
		super ("New Customer");
		CM = c;
		CM.addWindow ("CustEdit", (GridFrame) this);
		newCust = true;
		oldCust = null;
		oldName = null;
		myCust = new Customer ();
		theServer = s;
        badCustBG = new Color (211,137,127);
		goodCustBG = new Color (207,207,183);
		badCustInput = new Color (231,160,150);
		goodCustInput = Color.white;
		badCustFlag = false;
		initScreen ();
		parentList = l;
	}

	public CustEdit (CreateMenu c, Server s, CustTable l, Customer oc)
	{
		super ("Modify Customer Details");
		CM = c;
		if (CM != null)
			CM.addWindow ("CustEdit", (GridFrame) this);
		newCust = false;
		oldCust = oc;
		myCust = oldCust;
		oldName = myCust.name;
		theServer = s;
        badCustBG = new Color (211,137,127);
		goodCustBG = new Color (207,207,183);
		badCustInput = new Color (231,160,150);
		goodCustInput = Color.white;
		badCustFlag = oc.badCust;
        if (badCustFlag)
            setBackground (badCustBG);
		initScreen ();
		parentList = l;
	}

	public void initScreen ()
	{
		Label l_name = new Label ("Name:", Label.RIGHT);
		name = new InputField (30, InputField.CAPS);

		Label l_address = new Label ("Address:", Label.RIGHT);
		address = new InputField (30, InputField.TEXT);

		Label l_phone = new Label ("Phone:", Label.RIGHT);
		phone = new InputField (20, InputField.TEXT);

		Label l_suburb = new Label ("Suburb:", Label.RIGHT);
		suburb = new InputField (30, InputField.CAPS);

		Label l_minOrder = new Label ("Min:", Label.RIGHT);
		minOrder = new InputField (6, InputField.MONEY);

		Label l_comments = new Label ("Comments:", Label.RIGHT);
		comments = new CommentArea (5, 40);

		Label l_lastOrderNum = new Label ("Last Order Num:", Label.RIGHT);
		lastOrderNum = new InputField (10, InputField.INT);

		Label l_totalSpent = new Label ("Total Spent:", Label.RIGHT);
		totalSpent = new InputField (10, InputField.MONEY);

		Label l_lastOrderDate = new Label ("Last Order Date:", Label.RIGHT);
		lastOrderDate = new InputField (30, InputField.DATE);

		Label l_discount = new Label ("% Disc:", Label.RIGHT);
		discount = new InputField (3, InputField.INT);

		if (newCust == false)
		{
			name.setText (oldCust.name);
			address.setText (oldCust.address);
			phone.setText (oldCust.phone);
			suburb.setText (oldCust.suburb);
			Suburb mySuburb = theServer.getSuburb (oldCust.suburb);
			minOrder.setDouble (mySuburb.minOrder);
			comments.setText (oldCust.comments);
			lastOrderNum.setInt ((int)oldCust.lastOrderNum);
			totalSpent.setDouble (oldCust.totalSpent);
			lastOrderDate.setDate (oldCust.lastOrderDate);
			discount.setInt (oldCust.discount);
		}

		acceptBtn = new Button ("Accept");
			acceptBtn.setBackground (new Color (143, 200, 162));

 
        badCustBtn = new Button ("Bad Cust");
            badCustBtn.setBackground (new Color (255,128,128));

 
		cancelBtn = new Button ("Cancel");
			cancelBtn.setBackground (new Color (255,128,128));


		addElement (EAST, l_name, 1, 1, 1, 1);
		addElement (WEST, name,   2, 1, 1, 1);
		addElement (EAST, l_discount,   3, 1, 1, 1);
		addElement (WEST, discount,   4, 1, 1, 1);
		addElement (EAST, l_address,   1, 2, 1, 1);
		addElement (WEST, address,   2, 2, 0, 1);
		addElement (EAST, l_phone,   1, 3, 1, 1);
		addElement (WEST, phone,   2, 3, 0, 1);
		addElement (EAST, l_suburb,   1, 4, 1, 1);
		addElement (WEST, suburb,   2, 4, 1, 1);
		addElement (EAST, l_minOrder,   3, 4, 1, 1);
		addElement (WEST, minOrder,   4, 4, 0, 1);
		addElement (EAST, l_comments,   1, 5, 1, 1);
		addElement (WEST, comments,   2, 5, 0, 1);
		addElement (EAST, l_lastOrderNum,   1, 6, 1, 1);
		addElement (WEST, lastOrderNum,   2, 6, 0, 1);
		addElement (EAST, l_totalSpent,   1, 7, 1, 1);
		addElement (WEST, totalSpent,   2, 7, 0, 1);
		addElement (EAST, l_lastOrderDate,   1, 8, 1, 1);
		addElement (WEST, lastOrderDate,   2, 8, 0, 1);
		addElement (SOUTHEAST, acceptBtn,  2, 9, 1, 1);
		addElement (SOUTHEAST, badCustBtn,  3, 9, 1, 1);
		addElement (SOUTHEAST, cancelBtn,  4, 9, 1, 1);
		pack ();

		show ();
		name.requestFocus ();
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n')
		{
			if (evt.target == comments)
			{
				if (!evt.controlDown())
					return super.handleEvent (evt);
			}

			evt.id = Event.ACTION_EVENT;
			evt.target = acceptBtn;
			evt.arg = "Accept";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			if (CM != null)
				CM.removeWindow ((GridFrame) this);
			dispose ();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Cancel"))
		{
			if (CM != null)
				CM.removeWindow ((GridFrame) this);
			dispose ();
        } else if (evt.arg == "Bad Cust")
        {
            badCustFlag = !badCustFlag;
            Color iColor = goodCustInput;

            if (badCustFlag)
            {
                setBackground (badCustBG);
                iColor = badCustInput;
            }
            else
                setBackground (goodCustBG);

            myCust.badCust = badCustFlag;

            acceptBtn.setBackground (new Color (143, 200, 162));
            badCustBtn.setBackground (new Color (255,128,128));
            cancelBtn.setBackground (new Color (255,128,128));

            name.setBackground (iColor);
            address.setBackground (iColor);
            phone.setBackground (iColor);
            minOrder.setBackground (iColor);
            comments.setBackground (iColor);
            suburb.setBackground (iColor);

            comments.requestFocus();

		} else if (evt.arg == "Accept")
		{
			boolean doIt = true;

			if (newCust == false)
			{
				String c = name.getText();
				if (c.length() > 0 && myCust.name.equals(c) == false &&
					 theServer.custExists (c))
				{
					new MessageBox (this, "Error", "This customer name has already been used");
					doIt = false;
				}
			}
			if (address.getText().length() == 0)
			{
				address.requestFocus();
				doIt = false;
			}
			else if (phone.getText().length() == 0)
			{
				if (name.getText().length() == 0)
				{
					new MessageBox (this, "Error", "Must enter either a NAME or a PHONE number");
					phone.requestFocus();
					doIt = false;
				}
			}

			if (doIt)
			{
			myCust.name = name.getText();
			myCust.address = address.getText();
			myCust.phone = phone.getText();
			myCust.suburb = suburb.getText();
			myCust.comments = comments.getText();
			myCust.lastOrderNum = lastOrderNum.getInt();
			myCust.totalSpent = totalSpent.getDouble();
			myCust.lastOrderDate = lastOrderDate.getDate();
			myCust.discount = discount.getInt();

			theServer.setMinOrder (myCust.suburb, minOrder.getDouble());

			if (newCust)
			{
				theServer.newCust (myCust);
			}
			else
			{
				theServer.updateCust (myCust);
			}
			if (CM != null)
				CM.removeWindow ((GridFrame) this);
			dispose ();
			if (parentList != null)
				parentList.fillCustList();
			}
		}
		return (super.handleEvent (evt));
	}
}

class CustTableList extends List {

	public CustTableList ()
	{
		super (20, false);
	}

	public Dimension preferredSize ()
	{
		return new Dimension (300, 300);
	}
}

class CustTable extends GridFrame {

	CustTableList custList;
	InputField searchThis;
	Button searchBtn;
	Button addBtn;
	Button delBtn;
	Button modBtn;
	Button cancelBtn;
	Server theServer;
	CreateMenu CM;

	public CustTable (CreateMenu c, Server s)
	{
		super ("Customers");
		CM = c;
		CM.addWindow ("CustTable", (GridFrame) this);
		theServer = s;
		initScreen ();
	}

	public void initScreen ()
	{
		searchThis = new InputField (30, InputField.TEXT);

		custList = new CustTableList ();
		custList.setBackground (Color.white);
		fillCustList ();

		searchBtn = new Button ("Search");
			searchBtn.setBackground (new Color (145,200,162));

		addBtn = new Button ("Add");
			addBtn.setBackground (new Color (145,200,162));

		modBtn = new Button ("Modify");
			modBtn.setBackground (new Color (145,196,255));

		delBtn = new Button ("Delete");
			delBtn.setBackground (new Color (255,128,128));

		cancelBtn = new Button ("Close");
			cancelBtn.setBackground (new Color (255,128,128));

		addElement (NORTHWEST, searchThis, 1,1,1,1);

		gridc.fill = 1;
		addElement (NORTHWEST, custList, 1,2,1,0);

		gridc.ipadx = gridc.ipady = 20;
		addElement (SOUTHEAST, searchBtn,   2,1,1,1);
		addElement (SOUTHEAST, addBtn,   2,2,1,1);
		addElement (SOUTHEAST, modBtn,   2,3,1,1);
		addElement (SOUTHEAST, delBtn,   2,4,1,1);
		addElement (SOUTHEAST, cancelBtn,2,5,1,1);

		pack();
		show();
		custList.requestFocus ();
	}

	void fillCustList ()
	{
		custList.clear ();
		String filter = searchThis.getText();
		for (int i =0; i < theServer.theCustomers.size(); i++)
		{
			Customer myCust = (Customer) theServer.theCustomers.elementAt (i);
			if (filter.length() == 0 || (myCust.name.indexOf(filter) != -1 || 
				myCust.phone.startsWith(filter) ||
				myCust.address.indexOf(filter) != -1))
			custList.addItem (myCust.name + " " + myCust.phone + " " + myCust.suburb);
		}
	}

	Customer getCustAt (int i)
	{
		String filter = searchThis.getText();
		if (filter.length() == 0)
			return (Customer) theServer.theCustomers.elementAt (i);

		int iii = 0;
		for (int ii =0; ii < theServer.theCustomers.size(); ii++)
		{
			Customer myCust = (Customer) theServer.theCustomers.elementAt (ii);
			if (myCust.name.indexOf(filter) != -1 || 
				myCust.phone.startsWith(filter) ||
				myCust.address.indexOf(filter) != -1)
			{
				if (iii == i)
					return myCust;
				iii++;
			}
		}
		return null;
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n')
		{
			evt.id = Event.ACTION_EVENT;
			evt.target = modBtn;
			evt.arg = "Modify";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			CM.closeAllWindows ("CustEdit");
			CM.removeWindow ((GridFrame) this);
			dispose ();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Close"))
		{
			CM.closeAllWindows ("CustEdit");
			CM.removeWindow ((GridFrame) this);
			dispose ();
		} else if (evt.arg == "Search" || evt.key == '\n')
		{
			fillCustList ();
		} else if (evt.arg == "Add")
		{
			new CustEdit (CM, theServer, this);
		} else if (evt.arg == "Modify")
		{
			int i = custList.getSelectedIndex ();
			if (i != -1)
			{
			Customer myCust = getCustAt (i);
			new CustEdit (CM, theServer, this, myCust);
			}

		} else if ((evt.arg == "Delete") || (evt.id == Event.KEY_PRESS && evt.key == 127))
		{
			int i = custList.getSelectedIndex ();
			if (i != -1)
			{
			Customer myCust = getCustAt (i);
			theServer.deleteCust (myCust);
			fillCustList();
			}
		}

		return (super.handleEvent (evt));
	}
}
