/**********************************

InputField.java:	Support class for entering data



Date		Author		Ver		Comment
-------------------------------------------------------------
14 Sep 1997	SteveOC		1.0		Initial Version





**********************************/

import java.awt.*;
import java.util.Date;

public class InputField extends TextField {

	public static final int INT = 0;
	public static final int DATE = 1;
	public static final int TEXT = 2;
	public static final int CAPS = 4;
	public static final int PASSWORD = 8;
	public static final int PROTECTED = 16;
	public static final int MONEY = 32;

	int type;
	private boolean dirty;

	public InputField(int i, int t) {
		super(i);
		type = t;
		if ((type & PASSWORD) != 0)
			setEchoCharacter ('*');
		if ((type & PROTECTED) != 0)
			setBackground (new Color(207,207,183));
		dirty = false;
		}

	public boolean isChanged ()
	{
		return dirty;
	}

	public void isSet ()
	{
		dirty = false;
	}

	public boolean tabbable ()
	{
		if ((type & PROTECTED) != 0)
			return false;
		else
			return true;
	}

	public void setProtected (boolean newProt, int newType)
	{
		if (newProt)
		{
			type |= PROTECTED;
			setBackground (new Color (207,207,183));
		}
		else
		{
			type = newType;
			setBackground (Color.white);
		}
	}

	public void setText (String t)
	{
		dirty = false;
		if ((t.length() > 0) && ((type & MONEY) != 0))
		{
			double d = 0.0;
			
			try {
			new Double(t).doubleValue();
			} catch (Exception e)
			{
			}

			super.setText (Money.toString (d));
		}
		else
			super.setText (t);
	}

	public static String DateString (Date d)
	{
		if (d == null) return "";

		String s = d.toString();
		int i = s.indexOf (':');
		if (i != -1)
		{
			String s2 = s.substring (i+1);
			i += s2.indexOf (':');
		}
		String s3 = s.substring (0, i+1);
		i = s.lastIndexOf (' ');
		if (i != -1)
			s3 = s3 + s.substring (i);

		return s3;
	}

	public void setDate (Date d)
	{
		dirty = false;
		super.setText (DateString (d));
	}

	public void setInt (int i)
	{
		dirty = false;
		setText ("" + i);
	}

	public void setDouble (double d)
	{
		dirty = false;
		if ((type & MONEY) != 0)
		{
			super.setText (Money.toString (d));
		}
		else
			super.setText ("" + d);
	}

	public String getText ()
	{
		return super.getText().trim();
	}

	public double getDouble ()
	{
		if ((type == INT) || (type & MONEY) != 0)
		{
			double d = 0.0;
			try {
				String s = getText();
				if (s.length() > 0)
					d = new Double(s).doubleValue();
			} catch (Exception e)
			{
				System.out.println ("" + e + " " + getText());
				e.printStackTrace ();
			}
			return d;
		}
		return 0.0;
	}

	public int getInt ()
	{
		if ((type == INT) || ((type & MONEY) != 0))
		{
			int i = 0;
			try {
				i = Integer.parseInt (getText());
			} catch (Exception e)
			{
			}
			return i;
		}
		return 0;
	}

	public Date getDate()
	{
		if ((type & DATE) != 0)
		{
			Date d = null;
			try {
			d = new Date (getText());
			} catch (Exception e)
			{
			}
			return d;
		}
		return null;
	}

	public boolean handleEvent(Event e) {
		if ((e.id == Event.KEY_PRESS) && ((type & PROTECTED) != 0) &&
		    (e.key != '\t'))
		    {
		    return true;
		    }
		if ((e.id==Event.KEY_PRESS) && (e.key=='\n')) {
			return false;	// let parent eat it
			}
		if ((e.id==Event.KEY_PRESS) && ((type & CAPS) != 0)) {
			if ((e.key>='a')&&(e.key<='z')) e.key -= 'a' - 'A';
			}
		if ((e.id==Event.KEY_PRESS) && ((type & MONEY) != 0)) {
			if ((e.key>='a')&&(e.key<='z'))	return(true);
			if ((e.key>='A')&&(e.key<='Z'))	return(true);
		}
		if ((e.id==Event.KEY_PRESS) && (type==INT)) {
			if (e.key == 'q' || e.key == 'Q')
			{
				setText ("0.25");
			}
			else if (e.key == 'h' || e.key == 'H')
			{
				setText ("0.5");
			}
			else if (e.key == 'f' || e.key == 'F')
			{
				setText ("1.0");
			}
			else if (e.key == 'd' || e.key == 'D')
			{
				setText ("0.75");
			}
			if ((e.key>='a')&&(e.key<='z'))	return(true);
			if ((e.key>='A')&&(e.key<='Z'))	return(true);
			}	
							
		if (e.id == Event.KEY_PRESS)
		{
			if (e.key < 200 && e.key != 9)
				dirty = true;
		}

		return(super.handleEvent(e));
		}
}

			
