import java.io.*;
import java.awt.*;

class Item extends Object {

	String category, code, description;
	double price;

	public Item ()
	{
		category = null;
		code = null;
		description = null;
		price = 0.0;
	}

	public Item (DataInputStream fp, int Version) throws IOException
	{
		read (fp, Version);
	}

	public void read (DataInputStream in, int Version) throws IOException
	{
		category = in.readUTF();
		code = in.readUTF();
		description = in.readUTF();
		price = in.readDouble();
	}

	public void write (DataOutputStream out) throws IOException
	{
		out.writeUTF(category);
		out.writeUTF(code);
		out.writeUTF(description);
		out.writeDouble(price);
	}

	public void print (PrintStream out) throws IOException
	{
		String t = "\t";

		out.println (category + t + code + t + description + t + price);
	}

	public void printHeader (PrintStream out) throws IOException
	{
		out.println ("Category\tCode\tDescription\tPrice");
	}

	public void importString (String s)
	{
		String t = "\t";
		int i;

		i = s.indexOf (t);
		category = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		code = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		description = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		price = new Double(s).doubleValue();
	}
}

class ItemEdit extends GridFrame {

	InputField category, code, description, price;
	Button acceptBtn;
	Button cancelBtn;
	Server theServer;
	boolean newItem;
	Item oldItem;
	ItemTable parentList;
	CheckboxGroup itemType;
	Checkbox cbPizza, cbPasta, cbSteak, cbChicken, cbSeafood, cbDessert, cbSpecial, cbApp;
	Panel typePanel;
	CreateMenu CM;

	public ItemEdit (CreateMenu c, Server s, ItemTable l)
	{
		super ("New Item");
		CM = c;
		CM.addWindow ("ItemEdit", (GridFrame)this);
		newItem = true;
		oldItem = null;
		theServer = s;
		initScreen ();
		parentList = l;
	}

	public ItemEdit (CreateMenu c,Server s, ItemTable l, Item oi)
	{
		super ("Modify Item Details");
		CM = c;
		CM.addWindow ("ItemEdit", (GridFrame)this);
		newItem = false;
		oldItem = oi;
		theServer = s;
		initScreen ();
		parentList = l;
	}

	public void initScreen ()
	{
		itemType = new CheckboxGroup();
		typePanel = new Panel ();
			typePanel.add (cbPizza = new Checkbox ("Pizza", itemType, newItem));
			typePanel.add (cbPasta = new Checkbox ("Pasta", itemType, false));
			typePanel.add (cbSteak = new Checkbox ("Steak", itemType, false));
			typePanel.add (cbChicken = new Checkbox ("Chicken", itemType, false));
			typePanel.add (cbSeafood = new Checkbox ("Seafood", itemType, false));
			typePanel.add (cbDessert = new Checkbox ("Dessert", itemType, false));
			typePanel.add (cbSpecial = new Checkbox ("Special", itemType, false));
			typePanel.add (cbApp = new Checkbox ("Appetiser", itemType, false));

		Label l_code = new Label ("Item Code:", Label.RIGHT);
		code = new InputField (10, InputField.CAPS);

		Label l_description = new Label ("Description:", Label.RIGHT);
		description = new InputField (30, InputField.TEXT);

		Label l_price = new Label ("Price:", Label.RIGHT);
		price = new InputField (10, InputField.MONEY);

		if (newItem == false)
		{
			switch (oldItem.category.charAt(0))
			{
				case 'P': cbPizza.setState (true); break;
				case 'T': cbPasta.setState (true); break;
				case 'S': cbSteak.setState (true); break;
				case 'C': cbChicken.setState (true); break;
				case 'E': cbSeafood.setState (true); break;
				case 'D': cbDessert.setState (true); break;
				case 'Z': cbSpecial.setState (true); break;
				case 'A': cbApp.setState (true); break;
				default: cbPizza.setState (true); break;
			}

			code.setText (oldItem.code);
			description.setText (oldItem.description);
			price.setDouble (oldItem.price);
		}

		acceptBtn = new Button ("Accept");
			acceptBtn.setBackground (new Color (143, 200, 162));

		cancelBtn = new Button ("Cancel");
			cancelBtn.setBackground (new Color (255,128,128));


		gridc.fill = 1;
		addElement (NORTH, typePanel, 1,1,0,1);
		gridc.fill = 0;
		addElement (EAST, l_code, 1, 2, 1, 1);
		addElement (WEST, code,   2, 2, 0, 1);
		addElement (EAST, l_description,   1, 3, 1, 1);
		addElement (WEST, description,   2, 3, 0, 1);
		addElement (EAST, l_price,   1, 4, 1, 1);
		addElement (WEST, price,   2, 4, 0, 1);
		addElement (SOUTHEAST, acceptBtn,  2, 5, 1, 1);
		addElement (SOUTHEAST, cancelBtn,  4, 5, 1, 1);
		pack ();

		show ();
		code.requestFocus ();
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n')
		{
			evt.id = Event.ACTION_EVENT;
			evt.target = acceptBtn;
			evt.arg = "Accept";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			CM.removeWindow ((GridFrame)this);
			dispose ();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Cancel"))
		{
			CM.removeWindow ((GridFrame)this);
			dispose ();
		} else if (evt.arg == "Accept")
		{
			Item myItem;
			
			if (newItem)
				myItem = new Item ();
			else
				myItem = oldItem;

			Checkbox selected = itemType.getCurrent();
			myItem.category = "P";
			if (selected == cbPasta)
			{
				myItem.category = "T";
			} else if (selected == cbSteak)
			{
				myItem.category = "S";
			} else if (selected == cbChicken)
			{
				myItem.category = "C";
			} else if (selected == cbSeafood)
			{
				myItem.category = "E";
			} else if (selected == cbDessert)
			{
				myItem.category = "D";
			} else if (selected == cbSpecial)
			{
				myItem.category = "Z";
			} else if (selected == cbApp)
			{
				myItem.category = "A";
			}

			boolean doIt = true;

			if (newItem == false) 
			{
				String uc = code.getText();
				if (uc.length () == 0)
				{
					doIt = false;
					new MessageBox (this, "Error", "Cannot change item code to blank");
				}
				else if (myItem.code.equals(uc) == false &&
				    	 theServer.itemExists (uc))
				{
					new MessageBox (this, "Error", "This item already exists");
					doIt = false;
				}
			}
			
			if (doIt)
			{
				myItem.code = code.getText();
				myItem.description = description.getText();
				myItem.price = price.getDouble();
	
				if (newItem)
				{
					theServer.newItem (myItem);
				}
				else
				{
					theServer.updateItem (myItem);
				}
	
				CM.removeWindow ((GridFrame)this);
				dispose ();
				parentList.fillItemList();
			}
		}
		return (super.handleEvent (evt));
	}
}

class ItemTableList extends List {
	
	public ItemTableList (int rows)
	{
		super (rows, false);
	}

	public Dimension preferredSize ()
	{
		return (new Dimension (251, 251));
	}
}

class ItemTable extends GridFrame {

	InputField searchThis;
	ItemTableList itemList;
	Button searchBtn;
	Button addBtn;
	Button delBtn;
	Button modBtn;
	Button cancelBtn;
	Server theServer;
	CreateMenu CM;

	public ItemTable (CreateMenu c, Server s)
	{
		super ("Items");
		CM = c;
		CM.addWindow ("ItemTable", (GridFrame)this);
		theServer = s;
		initScreen ();
	}

	public void initScreen ()
	{
		searchThis = new InputField (30, InputField.TEXT);

		itemList = new ItemTableList (15);
		itemList.setBackground (Color.white);
		fillItemList ();

		searchBtn = new Button ("Search");
			searchBtn.setBackground (new Color (145,200,162));

		addBtn = new Button ("Add");
			addBtn.setBackground (new Color (145,200,162));

		modBtn = new Button ("Modify");
			modBtn.setBackground (new Color (145,196,255));

		delBtn = new Button ("Delete");
			delBtn.setBackground (new Color (255,128,128));

		cancelBtn = new Button ("Close");
			cancelBtn.setBackground (new Color (255,128,128));

		addElement (NORTHWEST, searchThis, 1,1,1,1);

		gridc.fill = 1;
		addElement (NORTHWEST, itemList, 1,2,1,0);

		gridc.ipadx = gridc.ipady = 20;
		addElement (SOUTHEAST, searchBtn,   2,1,1,1);
		addElement (SOUTHEAST, addBtn,   2,2,1,1);
		addElement (SOUTHEAST, modBtn,   2,3,1,1);
		addElement (SOUTHEAST, delBtn,   2,4,1,1);
		addElement (SOUTHEAST, cancelBtn,2,5,1,1);

		pack();
		show();
		itemList.requestFocus ();
	}

	void fillItemList ()
	{
		String filter = searchThis.getText();
		itemList.clear ();
		for (int i =0; i < theServer.theItems.size(); i++)
		{
			Item myItem = (Item) theServer.theItems.elementAt (i);
			if (filter.length() == 0 || myItem.description.indexOf (filter) != -1 ||
				myItem.code.startsWith (filter))
			itemList.addItem (myItem.category + "   " + myItem.code + " " + myItem.description + " $" + Money.toString(myItem.price));
		}
	}

	Item getItemAt (int i)
	{
		String filter = searchThis.getText();
		if (filter.length() == 0)
			return (Item) theServer.theItems.elementAt (i);

		int iii = 0;
		for (int ii =0; ii < theServer.theItems.size(); ii++)
		{
			Item myItem = (Item) theServer.theItems.elementAt (ii);
			if (myItem.code.startsWith(filter) || myItem.description.indexOf (filter) != -1)
			{
				if (iii == i)
					return myItem;
				iii++;
			}
		}
		return null;
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n' && evt.target == itemList)
		{
			evt.id = Event.ACTION_EVENT;
			evt.target = modBtn;
			evt.arg = "Modify";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			CM.closeAllWindows ("ItemEdit");
			CM.removeWindow ((GridFrame)this);
			dispose ();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Close"))
		{
			CM.closeAllWindows ("ItemEdit");
			CM.removeWindow ((GridFrame)this);
			dispose ();
		//} else if ((evt.arg == "Add") || (evt.id == Event.KEY_PRESS && evt.key == 'a'))
		} else if (evt.arg == "Add")
		{
			new ItemEdit (CM, theServer, this);
		} else if (evt.arg == "Search" || evt.key == '\n')
		{
			fillItemList ();
		} else if (evt.arg == "Modify")
		{
			int i = itemList.getSelectedIndex ();
			if (i > -1)
			{
			Item myItem = getItemAt (i);
			new ItemEdit (CM, theServer, this, myItem);
			}

		} else if ((evt.arg == "Delete") || (evt.id == Event.KEY_PRESS && evt.key == 127))
		{
			int i = itemList.getSelectedIndex ();
			if (i > -1)
			{
			Item myItem = getItemAt (i);
			theServer.deleteItem (myItem);
			fillItemList();
			}
		}

		return (super.handleEvent (evt));
	}
}

class ItemSelectTableList extends List {
	
	public ItemSelectTableList (int rows)
	{
		super (rows, true);
	}

	public Dimension preferredSize ()
	{
		return (new Dimension (351, 321));
	}
}

class ItemSelectTable extends GridFrame {

	ItemSelectTableList itemList;
	Button selBtn;
	Button cancelBtn;
	Server theServer;
	OrderEntry theParent;

	public ItemSelectTable (Server s, OrderEntry parent)
	{
		super ("Select Item");
		theServer = s;
		theParent = parent;
		initScreen ();
	}

	public void initScreen ()
	{
		itemList = new ItemSelectTableList (20);
		itemList.setBackground (Color.white);
		fillItemList ();

		selBtn = new Button ("Select");
			selBtn.setBackground (new Color (145,200,162));

		cancelBtn = new Button ("Close");
			cancelBtn.setBackground (new Color (255,128,128));

		gridc.fill = 1;
		addElement (NORTHWEST, itemList, 1,1,1,0);

		gridc.ipadx = gridc.ipady = 20;
		addElement (SOUTHEAST, selBtn,   2,4,1,1);
		addElement (SOUTHEAST, cancelBtn,2,5,1,1);

		pack();
		show();
		itemList.requestFocus ();
	}

	void fillItemList ()
	{
		itemList.clear ();
		for (int i =0; i < theServer.theItems.size(); i++)
		{
			Item myItem = (Item) theServer.theItems.elementAt (i);
			itemList.addItem (myItem.category + "   " + myItem.code + " " + myItem.description + " $" + Money.toString(myItem.price));
		}
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			dispose ();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Close"))
		{
			dispose ();
		} else if ((evt.id == Event.KEY_PRESS && evt.key == '\n') ||
			   (evt.arg == "Select"))
		{
			int i[] = itemList.getSelectedIndexes ();
			Vector retVal = new Vector (i.length);
			for (int j = 0; j < i.length; j++)
			{
				retVal.addElement (theServer.theItems.elementAt (i[j]));
			}
			theParent.setItems (retVal);
			dispose ();
		}

		return (super.handleEvent (evt));
	}
}

