import java.io.*;
import java.awt.*;
import java.util.Date;
import java.util.Vector;

class FutureDateEditor extends Dialog {
	
	Date retVal;
	InputField df;
	Button ok, cal, cancel;
	OrderEntry parent;

	FutureDateEditor (OrderEntry f, Date d)
	{
		super (f, "Set Future Date", true);
		parent = f;
		retVal = d;
		initscreen ();
	}

	String getMonth (int theMonth)
	{
		switch (theMonth + 1)
		{
			case 1: return "Jan";
			case 2: return "Feb";
			case 3: return "Mar";
			case 4: return "Apr";
			case 5: return "May";
			case 6: return "Jun";
			case 7: return "Jul";
			case 8: return "Aug";
			case 9: return "Sep";
			case 10: return "Oct";
			case 11: return "Nov";
			case 12: return "Dec";
		}
		return "Jan";
	}

	void initscreen ()
	{
		Panel p1 = new Panel();
			p1.add (new Label ("New Date :"));
			p1.add (df = new InputField (30, InputField.DATE)); 
		add ("North", p1);

		Label ll1 = new Label ("");
		Label ll2 = new Label ("Date Format: DD Mon YYYY HH:MM");
		ll2.setForeground (Color.blue);
		Label ll3 = new Label ("Time in 24hr format (ie 6pm = 18:00)");
		ll3.setForeground (Color.blue);
		Label ll4 = new Label ("");

		Panel pMiddle = new Panel();
		pMiddle.setLayout (new GridLayout (4,1));
			pMiddle.add (ll1);
			pMiddle.add (ll2);
			pMiddle.add (ll3);
			pMiddle.add (ll4);

		add ("Center", pMiddle);

		Panel p2 = new Panel ();
			p2.add (ok = new Button ("OK"));
			p2.add (cal = new Button ("Calendar"));
			p2.add (cancel = new Button ("Cancel"));
		add ("South", p2);

		// default the date fields to today
	
		if (retVal == null) retVal = new Date();

		String setThis = "" + retVal.getDate() + " " + getMonth (retVal.getMonth()) + " 19" +
			retVal.getYear () + " " + retVal.getHours() + ":" + retVal.getMinutes();
		df.setText (setThis);

		pack ();
		show ();

	}

	public boolean handleEvent (Event evt)
	{
		if ((evt.id == Event.KEY_PRESS && evt.key == 0x1b) ||
			(evt.arg == "Cancel"))
		{
			dispose ();
		} else if (evt.arg == "Calendar")
		{
			Date dateFrom = new Date();
			String command = "createcal 19" + dateFrom.getYear();
			try {
			Process p = Runtime.getRuntime().exec (command);
			p.waitFor ();
			new FileDisplay (parent, "Calendar View", "calendar");
			} catch (Exception ex)
			{
				ex.printStackTrace ();
			}
		} else if ((evt.id == Event.KEY_PRESS && evt.key == '\n') ||
				   (evt.arg == "OK"))
		{
			try {
				Date dateFrom = new Date (df.getText());

				System.out.println ("New Date " + dateFrom);
				parent.futureDate = dateFrom;
				dispose ();

			} catch (Exception e)
			{
				System.out.println ("Error: " + e);
				e.printStackTrace ();
				new MessageBox (parent, "Date Format Error", 
					"Please enter the date again in the format 'DD Mon YYYY HH:MM'");
				String setThis = "" + retVal.getDate() + " " + getMonth (retVal.getMonth()) + " 19" +
					retVal.getYear () + " " + retVal.getHours() + ":" + retVal.getMinutes();
				df.setText (setThis);
			}
		}
		return super.handleEvent (evt);
	}
}

class Order extends Object {

	long orderNum;
	boolean isPickup = false;
	String name; // the customer name
	String time; // pickup time
	long custID;
	Date date;
	Vector details;

	public Order ()
	{
		orderNum = 0;
		name = null;
		time = null;
		custID = -1;
		date = null;
		details = null;
		isPickup = false;
	}

	public void copyFrom (Order otherOrder)
	{
		name = otherOrder.name;
		time = otherOrder.time;
		custID = otherOrder.custID;
		isPickup = otherOrder.isPickup;

		details = new Vector ();
		for (int i = 0; i < otherOrder.details.size(); i++)
		{
			OrderDetail od = (OrderDetail) otherOrder.details.elementAt (i);
			System.out.println ("  copy " + i + od.code);
			details.addElement ( new OrderDetail (
				new String(od.code),
				new String (od.description), 
				od.qty, 
				od.price));
		}
	}

	public Order (DataInputStream fp, int Version) throws IOException
	{
		read (fp, Version);
	}

	public void read (DataInputStream in, int Version) throws IOException
	{
		orderNum = in.readLong ();
		if (Version >= 1)
		{
			isPickup = in.readBoolean ();
			if (isPickup)
			{
				name = in.readUTF();
				time = in.readUTF();
			}
			else
				custID = in.readLong ();
		}
		else
		{
			name = in.readUTF();
		}

		date = new Date (in.readLong());

		int count = in.readInt ();
		details = new Vector (count);
		for (int i = 0; i < count; i++)
		{
			OrderDetail myDet = new OrderDetail (in);
			details.addElement (myDet);
		}
	}

	public void write (DataOutputStream out) throws IOException
	{
		out.writeLong (orderNum);
		out.writeBoolean (isPickup);
		if (isPickup)
		{
			out.writeUTF (name);
			out.writeUTF (time);
		}
		else
			out.writeLong (custID);

		out.writeLong (date.getTime());

		int count = details.size();
		out.writeInt (count);

		for (int i = 0; i < count; i++)
		{
			OrderDetail myDet = (OrderDetail) details.elementAt (i);
			myDet.write (out);
		}
	}

	public double totalPrice ()
	{
		if (details == null) return 0.0;

		double retVal = 0.0;

		for (int i = 0; i < details.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) details.elementAt (i);
			retVal += myDet.price;
		}

		return retVal;
	}

	public double totalQty ()
	{
		if (details == null) return 0.0;

		double retVal = 0.0;

		for (int i = 0; i < details.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) details.elementAt (i);
			retVal += myDet.qty;
		}

		return retVal;
	}

	public void printDocket (Customer myCust)
	{
		try {

		//PrintStream fp = new PrintStream (new FileOutputStream ("/dev/lp1"));
		//PrintStream fp = new PrintStream (new FileOutputStream ("LPT1:"));
		String filename = "dockets/docket" + orderNum;
		PrintStream fp = new PrintStream (new FileOutputStream (filename));

		// this gets printed to a 'docket' file, and a shell script
		// adds the header , and spools it

		fp.println ("Order : " + orderNum);
		fp.println ("Date  : " + InputField.DateString (date));
		fp.println ("");
		if (isPickup)
		{
			fp.println ("Pickup\nName : " + name);
			fp.println ("Time : " + time);
			fp.println ("");
		}
		else
		{
			fp.println ("Name  : " + myCust.name);
			fp.println ("Addr  : " + myCust.address);
			fp.println ("        " + myCust.suburb);
			fp.println ("Phone : " + myCust.phone);
			fp.println ("");
			if (myCust.comments.length() > 0)
			{
				fp.println ("Comments :");
				fp.println (myCust.comments);
				fp.println ("");
			}
		}
		fp.println ("Description                          Price");
		fp.println ("==========================================");

		for (int i = 0; i < details.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) details.elementAt (i);
			String s = myDet.qty + " x " + myDet.description ;

			// special case for half
			if (myDet.code.equals ("HALF") || 
			    myDet.code.equals ("NO") ||
			    myDet.code.equals ("EXTRA") ||
			    myDet.code.equals ("DIS") ||
			    myDet.code.equals ("+") ||
			    myDet.code.equals ("--"))
				s = "    " + myDet.description;

			boolean done = false;

			while (!done)
			{
				while (s.length() < 34)
					s = s + " ";
				if (s.length () > 34)
				{
					fp.println (s.substring (0, 29) + " -");
					s = "  " + s.substring (29);
				}
				else
					done = true;
			}

			s = s + "$ ";
			String m = Money.toString (myDet.price);
			while (m.length() < 6)
				m = " " + m;
			fp.println (s + m);
		}
		double d = totalPrice();
		fp.println ("                                   -------");
			String m = Money.toString (d);
			while (m.length() < 6)
				m = " " + m;
		fp.println ("                          Total   $ " + m);
		fp.println ("");
		fp.println ("   Enjoy your meal !") ;
		fp.println ("");
		fp.println ("");
		fp.println ("");
		fp.println ("");
		fp.println ("");
		fp.println ("");
		fp.println ("");

		fp.flush ();
		fp.close ();

		Runtime.getRuntime().exec ("printit dockets/docket" + orderNum);

		} catch (IOException e)
		{
			e.printStackTrace ();
		}
		
	}
}

class OrderDetail extends Object {

	String code, description;
	double qty;
	double price;

	public OrderDetail ()
	{
		code = null;
		description = null;
		qty = 0.0;
		price = 0.0;
	}

	public OrderDetail (String c, String d, double q, double p)
	{
		code = c;
		description = d;
		qty = q;
		price = p;
	}

	public OrderDetail (DataInputStream fp) throws IOException
	{
		read (fp);
	}

	public void read (DataInputStream in) throws IOException
	{
		code = in.readUTF ();
		description = in.readUTF ();
		qty = in.readDouble ();
		price = in.readDouble ();
	}

	public void write (DataOutputStream out) throws IOException
	{
		out.writeUTF (code);
		out.writeUTF (description);
		out.writeDouble (qty);
		out.writeDouble (price);
	}
}

class OrderMenuCanvas extends Canvas {
	
	Image menu[] = new Image[10];
	int theMenu;
	MediaTracker m;
	Image MenuBar;

	public OrderMenuCanvas ()
	{
		m = new MediaTracker (this);
		theMenu = 0;

		try {
			for (int i = 0; i < 10; i++)
			{
				String imageName = "Menu" + (i+1) + ".gif";
				menu[i] = Toolkit.getDefaultToolkit().getImage (imageName);
				m.addImage (menu[i], i);
			}
			MenuBar = Toolkit.getDefaultToolkit().getImage ("MenuBar2.jpg");
			m.addImage (MenuBar, 10);
	
			m.waitForAll ();
		} catch (InterruptedException e)
		{
		}

		resize (320,370);
	}

	public void update (Graphics g)
	{
		paint (g);
	}

	public void setMenu (int i)
	{
		if (i < 0) i = 0;
		if (i > 9) i = 9;
		theMenu = i;
		repaint ();
	}

	public void paint (Graphics g)
	{
		if ((m.statusID(theMenu, false) & MediaTracker.ERRORED) != 0) {
			g.setColor (Color.gray);
			g.fillRect (0,0,320,320);
		}
		else
		{
			g.drawImage (menu [theMenu], 0, 0, this);	
			g.drawImage (MenuBar, 0, 320, this);
		}
	}

	public boolean mouseDown (Event evt, int x, int y)
	{
		if (evt.controlDown())
		{
			String cmd = "/usr/X11R6/bin/xpaint /java/pizza/Menu" + (theMenu+1) + ".gif";
			try {
				Runtime.getRuntime().exec (cmd);
				//String imageName = "Menu" + (theMenu+1) + ".gif";
				//menu[theMenu] = Toolkit.getDefaultToolkit().getImage (imageName);
				//m.addImage (menu[theMenu], 0);
				//m.waitForAll ();
				//repaint ();
			}
			catch (Exception e)
			{
				e.printStackTrace ();
			}
		}
		return super.mouseDown (evt, x, y);
	}
}

class TotalPriceDialog extends Dialog {

	OrderEntryPanel parentPanel;
	InputField newPrice;

	public TotalPriceDialog (Frame f, OrderEntryPanel p)
	{
		super (f, "Set Total Price", true);
		parentPanel = p;

		Panel p1 = new Panel ();
			p1.add (new Label ("Total Price:", Label.RIGHT));
			p1.add (newPrice = new InputField (10, InputField.MONEY));

		add ("South", new Button ("OK"));
		add ("North", p1);

		pack ();
		newPrice.requestFocus ();
		show ();
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.arg == "OK" || evt.key == '\n')
		{
			parentPanel.setNewTotal (newPrice.getDouble());
			dispose ();
			return false;
		}
		else if (evt.key == 0x1b)
		{
			dispose ();
			return false;
		}
		else
			return super.handleEvent (evt);
	}
}

class OrderEntryPanel extends GridPanel {

	Label l_code, l_description, l_qty, l_price;
	InputField code[];
	InputField description[];
	InputField qty[];
	InputField price[];
	Item myItem;
	Label totalPrice;

	Vector allRows;
	int scrollOffset;

	int CODE = 1;
	int DESC = 2;
	int QTY  = 3;
	int PRICE = 4;

	static int numFields = 12;

	Frame parentFrame;

	public OrderEntryPanel (Frame f)
	{
		super ();
		parentFrame = f;
		allRows = new Vector (20,10);
		scrollOffset = 0;
		initScreen ();
	}

	public void initScreen ()
	{
		l_code = new Label ("Code", Label.LEFT);
		l_description = new Label ("Description", Label.LEFT);
		l_qty = new Label ("Qty", Label.LEFT);
		l_price = new Label ("Price", Label.LEFT);

		code = new InputField[numFields];
		description = new InputField[numFields];
		qty = new InputField[numFields];
		price = new InputField[numFields];

		int i;
		Font myFont = new Font ("Dialog", Font.PLAIN, 11);
		for (i = 0; i < numFields; i++)
		{
			code[i] = new InputField (5, InputField.CAPS);
			description[i] = new InputField (25, InputField.TEXT);
			qty[i] = new InputField (4, InputField.INT);
			price[i] = new InputField (6, InputField.PROTECTED | InputField.MONEY);

			code[i].setFont(myFont);
			description[i].setFont(myFont);
			qty[i].setFont(myFont);
			price[i].setFont(myFont);
		}
		Font bigFont = new Font ("Dialog", Font.BOLD, 14);
		Button l_total = new Button ("Total Price:");
			l_total.setFont (bigFont);
		totalPrice = new Label ("$ 0.00", Label.RIGHT);
			totalPrice.setForeground (Color.blue);
			totalPrice.setFont (bigFont);


		addElement (NORTH, l_code, 1,1,1,1);
		addElement (NORTH, l_description, 2,1,1,1);
		addElement (NORTH, l_qty, 3,1,1,1);
		addElement (NORTH, l_price, 4,1,1,1);

		for (i = 0; i < numFields; i++)
		{
			addElement (NORTH, code[i], 1,2+i,1,1);
			addElement (NORTH, description[i], 2,2+i,1,1);
			addElement (NORTH, qty[i], 3,2+i,1,1);
			addElement (NORTH, price[i], 4,2+i,1,1);
		}

		addElement (SOUTH, l_total, 2, numFields + 2, 2, 1);
		addElement (SOUTH, totalPrice, 4, numFields + 2, 1, 1);

		code[0].requestFocus ();
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.arg == "Total Price:")
		{
			//System.out.println ("Pressed " + evt);
			new TotalPriceDialog (parentFrame, this);
		}
		else if (evt.id == Event.KEY_PRESS && evt.key == 9)
		{
			if (evt.target instanceof InputField)
			{
				InputField k = (InputField) evt.target;
				int theRow = fieldRow (k);
				int fieldType = editFieldType (k);
				if (fieldType == QTY && theRow == numFields-1)
				{
					screenToArray (theRow);
					scrollDown (1);
					code[numFields-1].requestFocus();
					return true;
				}
			}
			return super.handleEvent (evt);
		}
		if (evt.id == Event.KEY_PRESS && evt.key == 127)
		{
			InputField k = (InputField) evt.target;
			int theRow = fieldRow (k);
			int fieldType = editFieldType (k);

			if (fieldType == CODE)
			{
				code[theRow].setText ("");
				description[theRow].setText ("");
				qty[theRow].setText ("");
				price[theRow].setText ("");
			}
			screenToArray (theRow);
			recalcPrice();
		}
		else if (evt.id == Event.KEY_ACTION)
		{
			if ((evt.target instanceof InputField) == false)
				return super.handleEvent (evt);

			InputField k = (InputField) evt.target;
			int theRow = fieldRow (k);
			int fieldType = editFieldType (k);

			if (evt.key == Event.LEFT)
			{
				if (fieldType ==  CODE && theRow > 0)
					code[theRow-1].requestFocus();
				else if (fieldType == DESC)
					code[theRow].requestFocus();
				else if (fieldType == QTY)
					description[theRow].requestFocus();
			}
			if (evt.key == Event.RIGHT)
			{
				if (fieldType == CODE)
					description[theRow].requestFocus();
				else if (fieldType == DESC)
					qty[theRow].requestFocus(); 
				else if (fieldType == QTY && theRow < (numFields - 1))
					code[theRow+1].requestFocus(); 
			}

			if (evt.key == Event.UP)
			{
				if (theRow > 0)
				{
					if (fieldType == CODE)
						code[theRow-1].requestFocus();
					else if (fieldType == DESC)
						description[theRow-1].requestFocus();
					else if (fieldType == QTY)
						qty[theRow-1].requestFocus();
				}
				else // on top row
				{
					screenToArray (theRow);
					scrollUp (1);
				}
			}

			if (evt.key == Event.PGDN)
			{
				if (theRow < (numFields - 1))
				{
					code[numFields-1].requestFocus();
				}
				else
				{
					screenToArray (theRow);
					scrollDown (numFields);
				}
			}
			if (evt.key == Event.PGUP)
			{
				if (theRow > 0)
				{
					code[0].requestFocus();
				}
				else
				{
					scrollUp (numFields);
				}
			}
			if (evt.key == Event.HOME)
			{
				screenToArray (theRow);
				scrollOffset = 0;
				arrayToScreen ();
				code[0].requestFocus ();
			}
			if (evt.key == Event.END)
			{
				screenToArray (theRow);
				int allRowsSize = allRows.size();
				if (allRowsSize > numFields)
				{
					scrollOffset = allRowsSize - numFields;
					code[numFields-1].requestFocus();
				}
				else
				{
					scrollOffset = 0;
					code[allRowsSize-1].requestFocus();
				}
				arrayToScreen ();
			}

			if (evt.key == Event.DOWN)
			{
				if (theRow < (numFields - 1))
				{
					if (fieldType == CODE)
						code[theRow+1].requestFocus();
					else if (fieldType == DESC)
						description[theRow+1].requestFocus();
					else if (fieldType == QTY)
						qty[theRow+1].requestFocus();
				}
				else // bottom row
				{
					screenToArray (theRow);
					scrollDown (1);
				}
			}
		}
		return super.handleEvent (evt);
	}

	public void scrollUp (int i)
	{
		//System.out.println ("Scrolling up, currently " + scrollOffset);
		int oldOffset = scrollOffset;
		scrollOffset -= i;
		if (scrollOffset < 0) scrollOffset = 0;
		if (scrollOffset != oldOffset)
			arrayToScreen ();
	}

	public void scrollDown (int i)
	{
		//System.out.println ("Scrolling down, currently " + scrollOffset);
		int oldOffset = scrollOffset;
		scrollOffset += i;
		if (scrollOffset >= allRows.size()) scrollOffset = allRows.size() - 1;
			
		if (scrollOffset != oldOffset)
			arrayToScreen ();
	}

	public void recalcPrice ()
	{
		double thePrice = 0.0;

		for (int i = 0; i < allRows.size(); i++)
		{
			OrderDetail o = (OrderDetail) allRows.elementAt (i);
			if (o != null)
				thePrice += o.price;
		}

		totalPrice.setText ("$ " + Money.toString(thePrice));
	}

	public void setNewTotal (double d)
	{
		double thePrice = 0.0;
		OrderDetail o;

		for (int i = 0; i < allRows.size(); i++)
		{
			o = (OrderDetail) allRows.elementAt (i);
			if (o != null)
				thePrice += o.price;
		}

		double newDiscount = d - thePrice;
		if (newDiscount != 0.0)
		{
		System.out.println ("Apply new discount of " + newDiscount);

		o = new OrderDetail ();
		o.code = "DIS";
		o.description = "Discount";
		o.qty = 1.0;
		o.price = newDiscount;
		allRows.addElement (o);
		arrayToScreen ();
		}
		recalcPrice ();
		code[0].requestFocus();
	}

	public int editFieldType (InputField c)
	{
		for (int i = 0; i < numFields; i++)
		{
			if (code[i] == c)
				return CODE;
			else if (description[i] == c)
				return DESC;
			else if (qty[i] == c)
				return QTY;
			else if (price[i] == c)
				return PRICE;
		}
		return -1;
	}

	public int fieldRow (InputField c)
	{
		for (int i = 0; i < numFields; i++)
		{
			if (code[i] == c)
				return i;
			else if (description[i] == c)
				return i;
			else if (qty[i] == c)
				return i;
			else if (price[i] == c)
				return i;
		}
		return -1;
	}

	public void orderToArray (Order myOrder)
	{
		allRows.removeAllElements ();
		for (int i = 0; i < myOrder.details.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) myOrder.details.elementAt (i);
			OrderDetail o = new OrderDetail (myDet.code, myDet.description, myDet.qty, myDet.price);
			allRows.addElement (o);
		}
		//System.out.println ("orderToArray - " + allRows.size() + " elems");
		printRows ();
	}

	public void arrayToScreen ()
	{
		int allRowsSize = allRows.size();

		for (int i = 0; i < numFields; i++)
		{
			int r = i + scrollOffset;
			if (r < allRowsSize)
			{
				OrderDetail o = (OrderDetail) allRows.elementAt (r);
				if (o == null)
				{
					// have a null object in the array !
					code[i].setText ("");
					description[i].setText ("");
					qty[i].setText ("");
					price[i].setText ("");
				}
				else
				{
					// found an order for this line
					code[i].setText (o.code);
					description[i].setText (o.description);
					qty[i].setDouble (o.qty);
					price[i].setDouble (o.price);
				}
			}
			else // just clear it
			{
				code[i].setText ("");
				description[i].setText ("");
				qty[i].setText ("");
				price[i].setText ("");
			}
		}

		//System.out.println ("arrayToScreen");
		printRows ();
	}

	public void screenToArray (int row)
	{
		int i = row + scrollOffset;
		OrderDetail o = null;
		//System.out.println ("screenToArray " + row);
		int thisLength = code[row].getText().length();

		if (i < allRows.size())
		{
			o = (OrderDetail) allRows.elementAt (i);
			if (o == null && thisLength > 0)
			{
				// create a new one if it is not blank
				o = new OrderDetail (code[row].getText(),
					description[row].getText(),
					qty[row].getDouble(),
					price[row].getDouble());
				allRows.setElementAt (o, i);
			}
			else
			{
				// change the existing one
				if (thisLength == 0)
					o = null;
				else
				{
					o.code = code[row].getText();
					o.description = description[row].getText();
					o.qty = qty[row].getDouble();
					o.price = price[row].getDouble();
				}

				allRows.setElementAt (o, i);
			}
		}
		else if (thisLength > 0) // is a new one and it is not blank
		{
			int oldSize = allRows.size();
			allRows.setSize(i+1);
			// set all the new ones to null
			for (int newUn = oldSize; newUn <= i; newUn++)
				allRows.setElementAt (null, newUn);

			o = new OrderDetail (code[row].getText(),
				description[row].getText(),
				qty[row].getDouble(),
				price[row].getDouble());

			allRows.setElementAt (o, i);
		}

		//System.out.println ("size = " + allRows.size());
		printRows ();
	}

	public void printRows ()
	{
		return;
		/*
		for (int i = 0; i < allRows.size(); i++)
		{
			OrderDetail o = (OrderDetail) allRows.elementAt (i);
			if (o == null)
			System.out.println ("  " + i + " is null");
			else
			System.out.println ("  " + i + o.code + " " + o.description + " " + o.qty + " " + o.price);
		}
		*/
		//Thread.currentThread().dumpStack();
	}
}

class OrderEntry extends GridFrame {

	Order myOrder;
	Server theServer;
	boolean newOrder;
	Button acceptNoPrintBtn, acceptBtn, cancelBtn, closeBtn;
	OrderMenuCanvas menuPic;
	OrderEntryPanel orderPanel;
	String promptString1, promptString2;
	InputField orderName, orderTime;
	Select theParent;
	Customer myCust;
	double oldPrice;
	int onRow;
	Color focusColor;
	String theName, theTime;
	Date futureDate = null;

	Item myItem;

	int CODE = 1;
	int DESC = 2;
	int QTY  = 3;
	int PRICE = 4;

	// constructor for orders that are a 'same again please' repeat of a preivous order

	public OrderEntry (Select ss, Customer c, Server s, long CopyFrom)
	{
		super ("Repeat Order");
		theParent = ss;
		theName = null;
		theTime = null;
		theServer = s;
		myOrder = new Order ();

		Order oldOrder = theServer.getOrder (CopyFrom);
		if (oldOrder != null)
		{
			myOrder.copyFrom (oldOrder);
		}

		System.out.println ("myorder copied, now has " + myOrder.details.size() + " items $" + Money.toString (myOrder.totalPrice()));

		myCust = c;
		if (myCust == null)
		{
			myOrder.isPickup = true;
		}
		else
		{
			promptString1 = "Customer: " + myCust.name + ", " + myCust.address;
			promptString2 = "Delivery to " + myCust.suburb + " UBD " + myCust.ubd + " Min Order " + myCust.minOrder;
			myOrder.custID = myCust.ID;
		}

		newOrder = true;
		myItem = null;
		initScreen ();
		orderToScreen ();
	}

	public OrderEntry (Select ss, Customer c, Server s)
	{
		super ("New Order Details");
		theParent = ss;
		theName = null;
		theTime = null;
		theServer = s;
		myOrder = new Order ();
		myCust = c;
		if (myCust == null)
		{
			myOrder.isPickup = true;
		}
		else
		{
			promptString1 = "Customer: " + myCust.name + ", " + myCust.address;
			promptString2 = "Delivery to " + myCust.suburb + " UBD " + myCust.ubd + " Min Order " + myCust.minOrder;
			myOrder.custID = myCust.ID;
		}

		newOrder = true;
		myItem = null;
		initScreen ();
	}

	public OrderEntry (Select ss, Customer c, Server s, Order o)
	{
		super ("Old Order Details");
		theParent = ss;
		theServer = s;
		myOrder = o;
		myCust = c;
		if (myCust != null)
		{
			promptString1 = "Customer: " + myCust.name + ", " + myCust.address;
			promptString2 = "Delivery to " + myCust.suburb + " UBD " + myCust.ubd + " Min Order " + myCust.minOrder;
			myOrder.custID = myCust.ID;
		}

		oldPrice = myOrder.totalPrice();
		newOrder = false;
		myItem = null;
		initScreen ();
		orderToScreen ();
	}

	public void makeCopy (long copyThisOrder)
	{
		Order otherOrder = theServer.getOrder (copyThisOrder);
		if (otherOrder != null)
		{
			myOrder.details = new Vector ();
			for (int i = 0; i < otherOrder.details.size(); i++)
			{
				OrderDetail od = (OrderDetail) otherOrder.details.elementAt (i);
				myOrder.details.addElement ( new OrderDetail (
					new String(od.code),
					new String (od.description), 
					od.qty, 
					od.price));
			}
			orderToScreen ();
		}
	}

	public void initScreen ()
	{
		Label l_prompt, l_prompt2;
		if (myOrder.isPickup)
		{
			l_prompt = new Label ("Pickup By:", Label.RIGHT);
			orderName = new InputField (30, InputField.TEXT);
			l_prompt2 = new Label ("Pickup Time:", Label.RIGHT);
			orderTime = new InputField (15, InputField.CAPS);

			if (newOrder)	// then default time to now + 30 minutes
			{
				Date d = new Date();
				int h = d.getHours();
				int m = d.getMinutes() + 30;

				// round minutes up to nearest 5 minutes
				int spareMinutes = m % 5;
				if (spareMinutes != 0)
				{
					m -= spareMinutes;
					m += 5;
				}

				if (m >= 60)
				{
					m -= 60;
					h ++;
				}
				if (h >= 24)
				{
					h -= 24;
				}
				double dbl = (h * 1.0) + (m / 100.0);
				orderTime.setText (Money.toString (dbl));
			}
		}
		else
		{
			focusColor = new Color (130,252,237);
			l_prompt = new Label (promptString1, Label.CENTER);
			l_prompt.setForeground (new Color(0,0,64));
			l_prompt2 = new Label (promptString2, Label.CENTER);
			l_prompt2.setForeground (Color.blue);
		}

		orderPanel = new OrderEntryPanel (this);

		menuPic = new OrderMenuCanvas ();

		acceptBtn = new Button ("Accept");
			acceptBtn.setBackground (new Color (145,200,162));

		acceptNoPrintBtn = new Button ("Accept - Dont Print");
			acceptNoPrintBtn.setBackground (new Color (145,200,162));

		if (newOrder == false)
		{
			cancelBtn = new Button ("Cancel Order");
				cancelBtn.setBackground (new Color (255,128,128));
		}

		closeBtn = new Button ("Close");
			closeBtn.setBackground (new Color (145,196,255));

		gridc.fill = GridBagConstraints.HORIZONTAL;
		gridc.ipadx = gridc.ipady = 0;
		gridc.weightx = gridc.weighty = 0.0;

		if (myOrder.isPickup)
		{
			addElement (NORTH, l_prompt, 1,1,1,1);
			addElement (NORTH, orderName, 2,1,1,1);
			addElement (NORTH, l_prompt2, 1,2,1,1);
			addElement (NORTH, orderTime, 2,2,1,1);
		}
		else
		{
			addElement (NORTH, l_prompt, 1,1,1,1);
			addElement (NORTH, l_prompt2, 1,2,1,1);
		}

		gridc.fill = 0;
		addElement (NORTHWEST, orderPanel, 1,3,2,1);

		Panel pix = new Panel();
			pix.setLayout (new BorderLayout());
			pix.add ("Center", menuPic);
			if (myOrder.isPickup == false)
			{
				Panel btnPanel = new Panel();
				Button altC = new Button ("Cust Details");
				altC.setForeground (Color.red);
				Button lastOrd = new Button ("Last Orders");
				lastOrd.setForeground (Color.blue);
				Button FutureDate = new Button ("Future Date");
				FutureDate.setForeground (Color.blue);
				btnPanel.add (altC);
				btnPanel.add (lastOrd);
				btnPanel.add (FutureDate);
				pix.add ("South", btnPanel);
			}

		addElement (NORTHEAST, pix, 3,1,3,3);

		gridc.ipadx = gridc.ipady = 20;
		addElement (SOUTHEAST, acceptBtn,   2,4,1,1);
		addElement (SOUTHEAST, acceptNoPrintBtn,   3,4,1,1);

		if (newOrder == false)
			addElement (SOUTHEAST, cancelBtn,   4,4,1,1);

		addElement (SOUTHEAST, closeBtn,   5,4,1,1);

		pack();
		show();
			
		if (myOrder.isPickup && orderName.getText().length() == 0)
		{
			orderName.requestFocus ();
		}
		else
			orderPanel.code[0].requestFocus();
	}

	void orderToScreen ()
	{
		if (myOrder == null) { System.out.println ("null order"); return; }
		if (myOrder.details == null) { System.out.println ("null det"); return; }

		if (myOrder.isPickup)
		{
			orderName.setText (myOrder.name);
			orderTime.setText (myOrder.time);
		}

		orderPanel.orderToArray (myOrder);
		orderPanel.arrayToScreen ();
		orderPanel.recalcPrice();
		orderPanel.code[0].requestFocus();
	}

	void screenToOrder ()
	{
		if (newOrder && (myOrder.orderNum == 0))
		{
			myOrder.orderNum = theServer.newOrderNum();
		}
		if (myCust != null)
			myOrder.custID = myCust.ID;

		if (myOrder.isPickup)
		{
			myOrder.name = orderName.getText();
			myOrder.time = orderTime.getText();
		}

		if (futureDate == null)
		{
			if (myOrder.date == null) myOrder.date = new Date();
		}
		else
			myOrder.date = futureDate;

		myOrder.details = new Vector ();

		//System.out.println ("Screen to order");
		orderPanel.printRows ();
		for (int i = 0; i < orderPanel.allRows.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) orderPanel.allRows.elementAt (i);
			if (myDet != null)
				myOrder.details.addElement (myDet);
		}
	}

	int editFieldType (InputField c)
	{
		return orderPanel.editFieldType (c);
	}

	int fieldRow (InputField c)
	{
		return orderPanel.fieldRow (c);
	}

	public boolean handleEvent (Event evt)
	{
		if ((evt.arg == "Cust Details") ||
			(evt.metaDown() && evt.id == Event.KEY_PRESS && evt.key == 'C') ||
			(evt.metaDown() && evt.id == Event.KEY_PRESS && evt.key == 'c'))
		{
			if (myOrder.isPickup == false)
			{
				new CustEdit (null, theServer, null, myCust);
				return (true);
			}
		}
		if ((evt.arg == "Last Orders") || 
			 (evt.metaDown() && evt.id == Event.KEY_PRESS && evt.key == 'L') ||
			 (evt.metaDown() && evt.id == Event.KEY_PRESS && evt.key == 'l'))
		{
			if (myOrder.isPickup == false)
			{
				new LastOrders (this, theServer, myCust);
				return (true);
			}
		}
		if ((evt.arg == "Future Date") || 
			 (evt.metaDown() && evt.id == Event.KEY_PRESS && evt.key == 'F') ||
			 (evt.metaDown() && evt.id == Event.KEY_PRESS && evt.key == 'F'))
		{
			System.out.println ("Debug 1");
			if (myOrder.isPickup == false)
			{
			System.out.println ("Debug 2");
				new FutureDateEditor (this, futureDate);
			System.out.println ("Debug 3");
				System.out.println ("Future Date = " + futureDate);
				return (true);
			}
		}

		if (evt.id == Event.KEY_PRESS && evt.key == '\n')
		{
			int theField = editFieldType ((InputField) evt.target);
			int theRow = fieldRow ((InputField) evt.target);

			boolean OK = false;

			if (theRow != -1)
			{
			orderPanel.screenToArray (theRow);
			if (orderPanel.code[theRow].getText().length() == 0)
			{
				OK = true;
			}
			else if (orderPanel.description[theRow].getText().length() != 0)
			{
				OK = true;
			}

			if (OK)
			{
				myItem = theServer.getItem (orderPanel.code[theRow].getText());
				if (myItem != null)
				{
					double v = myItem.price * orderPanel.qty[theRow].getDouble();
					if (myCust != null && myCust.discount != 0 && myItem.category.equals("Z") == false)
						v *= ((100-myCust.discount)/100.0);
					orderPanel.price[theRow].setDouble (v);
				}
				evt.id = Event.ACTION_EVENT;
				evt.target = acceptBtn;
				evt.arg = "Accept";
				super.deliverEvent (evt);
				return (true);
			}
			}
		}
		else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Close"))
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.arg == "Cancel Order")
		{
			if (oldPrice > 0.0)
			{
				myOrder.details = new Vector (); // wipe out details
				if (myCust != null)
					myCust.totalSpent -= oldPrice;

				if (myOrder.isPickup)
				{
					myOrder.name = "** Cancelled ** " + myOrder.name;
				}
			}
			theServer.updateOrder (myOrder, oldPrice);
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.arg == "Accept" || evt.arg == "Accept - Dont Print")
		{
			// update the database
			screenToOrder ();
			double d = myOrder.totalPrice();

			if (newOrder)
			{
				if (d > 0.0)
				{
					if (myCust != null)
					{
						myCust.lastOrderNum = myOrder.orderNum;
						myCust.totalSpent += myOrder.totalPrice();
						myCust.lastOrderDate = myOrder.date;
					}
					theServer.newOrder (myOrder);
				}
			} else {
				if (myCust != null)
				{
					myCust.totalSpent -= oldPrice;
					myCust.totalSpent += myOrder.totalPrice();
				}
				theServer.updateOrder (myOrder, oldPrice);
			}
			// print the receipt

			if (evt.arg == "Accept")
				myOrder.printDocket(myCust);

			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.id == Event.KEY_ACTION)
		{
			switch (evt.key)
			{
				case Event.F1: menuPic.setMenu (0); break;
				case Event.F2: menuPic.setMenu (1); break;
				case Event.F3: menuPic.setMenu (2); break;
				case Event.F4: menuPic.setMenu (3); break;
				case Event.F5: menuPic.setMenu (4); break;
				case Event.F6: menuPic.setMenu (5); break;
				case Event.F7: menuPic.setMenu (6); break;
				case Event.F8: menuPic.setMenu (7); break;
				case Event.F9: menuPic.setMenu (8); break;
				case Event.F10: menuPic.setMenu (9); break;
				case Event.F12:
					if (evt.target instanceof InputField)
					{
						new ItemSelectTable (theServer, this);
					}
					break;
			}
		} else if (evt.id == Event.GOT_FOCUS)
		{
			if (evt.target instanceof InputField)
			{
				((InputField)evt.target).setBackground (focusColor);
				int theField = editFieldType ((InputField) evt.target);
				int theRow = fieldRow ((InputField) evt.target);

				if (theRow != -1)
				{
					onRow = theRow;
		
					if (theField == CODE || theField == QTY || theField==PRICE)
					{
						InputField c = (InputField) evt.target;
						c.select(0,100);
					}

					// if gain focus on the very last qty on the screen
					// 99% of the time this will be because they have tabbed from
					// the desc field, so we scroll the screen down by 1 at this point if

					//if (theField == QTY && theRow == (orderPanel.numFields-1))
					//{
						//orderPanel.scrollDown (1);
						////System.out.println ("set focus to " + (theRow-1));
						//orderPanel.qty[theRow-1].requestFocus();
					//}
				}
			}
		} else if (evt.id == Event.LOST_FOCUS)
		{
			if (evt.target instanceof InputField)
			{
				((InputField)evt.target).setBackground (Color.white);
				int theField = editFieldType ((InputField) evt.target);
				int theRow = fieldRow ((InputField) evt.target);
		
				if (theRow != -1)
				{
					processFocusLost (theField, theRow);
					orderPanel.screenToArray (theRow);
					orderPanel.recalcPrice();
				}
			}
		}

		return (super.handleEvent (evt));
	}

	public void setItems (Vector myItems)
	{
		if (onRow != -1)
		{
			int i;
			for (i = 0; i < myItems.size(); i++)
			{
				if ((onRow + i) > 9) break;

				Item myItem = (Item) myItems.elementAt (i);
				orderPanel.code[onRow+i].setText (myItem.code);
				orderPanel.description[onRow+i].setText (myItem.description);
				double v = orderPanel.qty[onRow+i].getDouble();
				if (v == 0.0)
				{
					orderPanel.qty[onRow+i].setText ("1.0");
					orderPanel.price[onRow+i].setDouble (myItem.price);
				}
				else {
					v *= myItem.price;
					if (myCust != null && myCust.discount != 0 && myItem.category.equals("Z") == false)
						v *= ((100-myCust.discount)/100.0);
					orderPanel.price[onRow+i].setDouble (v);
				}
				orderPanel.screenToArray (onRow+i);
			}
			orderPanel.recalcPrice();
			i --;
			if ((onRow + i) > 9)
				orderPanel.code[9].requestFocus ();
			else
				orderPanel.code[onRow + i].requestFocus ();
		}
	}

	public void processFocusLost (int theField, int theRow)
	{
		if (theField != -1 && theRow != -1)
		{
			if (theField == QTY)
			{
				if (orderPanel.qty[theRow].isChanged())
				{
				myItem = theServer.getItem (orderPanel.code[theRow].getText());
				orderPanel.qty[theRow].isSet();
				if (myItem != null)
				{
					double v = myItem.price * orderPanel.qty[theRow].getDouble();
					if (myCust != null && myCust.discount != 0 && myItem.category.equals("Z") == false)
						v *= ((100 - myCust.discount)/100.0);
					orderPanel.price[theRow].setDouble (v);
				}
				}
			}
			else if (theField == CODE)
			{
				if (orderPanel.code[theRow].isChanged())
				{
				orderPanel.code[theRow].isSet();
				myItem = theServer.getItem (orderPanel.code[theRow].getText());
				if (myItem != null)
				{ 
					//if ((orderPanel.description[theRow].getText().length() == 0))
					orderPanel.description[theRow].setText (myItem.description);

					if (myItem.code.equals ("CIGS") ||
					    myItem.code.equals ("+") ||
					    myItem.code.equals ("DIS"))
					{
						orderPanel.price[theRow].setProtected (false, InputField.MONEY);
					}
					else
					{
						orderPanel.price[theRow].setProtected (true, InputField.MONEY);
					}

					double v = orderPanel.qty[theRow].getDouble();
					if (v == 0.0)
					{
						orderPanel.qty[theRow].setText ("1.0");
						v = myItem.price;
						if (myCust != null && myCust.discount != 0 && myItem.category.equals("Z") == false)
							v *= ((100 - myCust.discount)/100.0);
						orderPanel.price[theRow].setDouble (v);
					}
					else {
						v *= myItem.price;
						if (myCust != null && myCust.discount != 0 && myItem.category.equals("Z") == false)
							v *= ((100 - myCust.discount)/100.0);
						orderPanel.price[theRow].setDouble (v);
					}
				}
				else {
					orderPanel.description[theRow].setText ("");
					orderPanel.qty[theRow].setText ("");
					orderPanel.price[theRow].setText ("");
				}
				}
			}
		}
	}

}

