import java.io.*;
import java.awt.*;
import java.util.Date;
import java.util.Vector;

class Order extends Object {

	long orderNum;
	boolean isPickup = false;
	String name; // the customer name
	String time; // pickup time
	long custID;
	Date date;
	Vector details;

	public Order ()
	{
		orderNum = 0;
		name = null;
		time = null;
		custID = -1;
		date = null;
		details = null;
		isPickup = false;
	}

	public Order (DataInputStream fp, int Version) throws IOException
	{
		read (fp, Version);
	}

	public void read (DataInputStream in, int Version) throws IOException
	{
		orderNum = in.readLong ();
		if (Version >= 1)
		{
			isPickup = in.readBoolean ();
			if (isPickup)
			{
				name = in.readUTF();
				time = in.readUTF();
			}
			else
				custID = in.readLong ();
		}
		else
		{
			name = in.readUTF();
		}

		date = new Date (in.readLong());

		int count = in.readInt ();
		details = new Vector (count);
		for (int i = 0; i < count; i++)
		{
			OrderDetail myDet = new OrderDetail (in);
			details.addElement (myDet);
		}
	}

	public void write (DataOutputStream out) throws IOException
	{
		out.writeLong (orderNum);
		out.writeBoolean (isPickup);
		if (isPickup)
		{
			out.writeUTF (name);
			out.writeUTF (time);
		}
		else
			out.writeLong (custID);

		out.writeLong (date.getTime());

		int count = details.size();
		out.writeInt (count);

		for (int i = 0; i < count; i++)
		{
			OrderDetail myDet = (OrderDetail) details.elementAt (i);
			myDet.write (out);
		}
	}

	public double totalPrice ()
	{
		if (details == null) return 0.0;

		double retVal = 0.0;

		for (int i = 0; i < details.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) details.elementAt (i);
			retVal += myDet.price;
		}

		return retVal;
	}

	public void printDocket (Customer myCust)
	{
		try {

		//PrintStream fp = new PrintStream (new FileOutputStream ("/dev/lp1"));
		//PrintStream fp = new PrintStream (new FileOutputStream ("LPT1:"));
		String filename = "docket";
		PrintStream fp = new PrintStream (new FileOutputStream (filename));

		fp.println ("       San Giorgio");
		fp.println ("");
		fp.println ("    217 Rundle Street");
		fp.println ("       Adelaide 5000");
		fp.println ("   Phone (08) 8223 1700");
		fp.println ("   Fax   (08) 8232 3500");
		fp.println ("");
		fp.println ("Order : " + orderNum);
		fp.println ("Date  : " + InputField.DateString (date));
		fp.println ("");
		if (isPickup)
		{
			fp.println ("Pickup\nName : " + name);
			fp.println ("Time : " + time);
			fp.println ("");
		}
		else
		{
			fp.println ("Name  : " + myCust.name);
			fp.println ("Addr  : " + myCust.address);
			fp.println ("        " + myCust.suburb);
			fp.println ("Phone : " + myCust.phone);
			fp.println ("");
			if (myCust.comments.length() > 0)
			{
				fp.println ("Comments :");
				fp.println (myCust.comments);
				fp.println ("");
			}
		}
		fp.println ("Description                          Price");
		fp.println ("==========================================");

		for (int i = 0; i < details.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) details.elementAt (i);
			String s = myDet.qty + " x " + myDet.description ;

			// special case for half
			if (myDet.code.equals ("HALF"))
				s = "    " + myDet.description;

			// special case for extra
			if (myDet.description.startsWith ("EXTRA "))
				s = "    " + myDet.description;

			boolean done = false;

			while (!done)
			{
				while (s.length() < 35)
					s = s + " ";
				if (s.length () > 35)
				{
					fp.println (s.substring (0, 30) + " -");
					s = "  " + s.substring (30);
				}
				else
					done = true;
			}

			s = s + "$ ";
			String m = Money.toString (myDet.price);
			if (m.length() == 4)
				s = s + " ";
			fp.println (s + m);
		}
		double d = totalPrice();
		fp.println ("                                   -------");
		fp.println ("                           Total   $ " + Money.toString(d));
		fp.println ("");
		fp.println ("   Enjoy your meal !") ;
		fp.println ("");
		fp.println ("");
		fp.println ("");
		fp.println ("");

		fp.flush ();
		fp.close ();

		Runtime.getRuntime().exec ("printit");

		} catch (IOException e)
		{
			e.printStackTrace ();
		}
		
	}
}

class OrderDetail extends Object {

	String code, description;
	double qty;
	double price;

	public OrderDetail ()
	{
		code = null;
		description = null;
		qty = 0.0;
		price = 0.0;
	}

	public OrderDetail (DataInputStream fp) throws IOException
	{
		read (fp);
	}

	public void read (DataInputStream in) throws IOException
	{
		code = in.readUTF ();
		description = in.readUTF ();
		qty = in.readDouble ();
		price = in.readDouble ();
	}

	public void write (DataOutputStream out) throws IOException
	{
		out.writeUTF (code);
		out.writeUTF (description);
		out.writeDouble (qty);
		out.writeDouble (price);
	}
}

class OrderMenuCanvas extends Canvas {
	
	Image menu[] = new Image[10];
	int theMenu;
	MediaTracker m;
	Image MenuBar;

	public OrderMenuCanvas ()
	{
		m = new MediaTracker (this);
		theMenu = 0;

		try {
			for (int i = 0; i < 10; i++)
			{
				String imageName = "Menu" + (i+1) + ".gif";
				menu[i] = Toolkit.getDefaultToolkit().getImage (imageName);
				m.addImage (menu[i], i);
			}
			MenuBar = Toolkit.getDefaultToolkit().getImage ("MenuBar2.jpg");
			m.addImage (MenuBar, 10);
	
			m.waitForAll ();
		} catch (InterruptedException e)
		{
		}

		resize (320,370);
	}

	public void update (Graphics g)
	{
		paint (g);
	}

	public void setMenu (int i)
	{
		if (i < 0) i = 0;
		if (i > 9) i = 9;
		theMenu = i;
		repaint ();
	}

	public void paint (Graphics g)
	{
		if ((m.statusID(theMenu, false) & MediaTracker.ERRORED) != 0) {
			g.setColor (Color.gray);
			g.fillRect (0,0,320,320);
		}
		else
		{
			g.drawImage (menu [theMenu], 0, 0, this);	
			g.drawImage (MenuBar, 0, 320, this);
		}
	}
}

class OrderEntryPanel extends GridPanel {

	Label l_code, l_description, l_qty, l_price;
	InputField code[];
	InputField description[];
	InputField qty[];
	InputField price[];
	Item myItem;
	Label totalPrice;

	int CODE = 1;
	int DESC = 2;
	int QTY  = 3;
	int PRICE = 4;

	public OrderEntryPanel ()
	{
		super ();
		initScreen ();
	}

	public void initScreen ()
	{
		l_code = new Label ("Code", Label.LEFT);
		l_description = new Label ("Description", Label.LEFT);
		l_qty = new Label ("Qty", Label.LEFT);
		l_price = new Label ("Price", Label.LEFT);

		code = new InputField[12];
		description = new InputField[12];
		qty = new InputField[12];
		price = new InputField[12];

		int i;
		Font myFont = new Font ("Dialog", Font.PLAIN, 11);
		for (i = 0; i < 12; i++)
		{
			code[i] = new InputField (5, InputField.CAPS);
			description[i] = new InputField (25, InputField.TEXT);
			qty[i] = new InputField (4, InputField.INT);
			price[i] = new InputField (6, InputField.PROTECTED | InputField.MONEY);

			code[i].setFont(myFont);
			description[i].setFont(myFont);
			qty[i].setFont(myFont);
			price[i].setFont(myFont);
		}
		Font bigFont = new Font ("Dialog", Font.BOLD, 14);
		Label l_total = new Label ("Total Price:", Label.RIGHT);
			l_total.setFont (bigFont);
		totalPrice = new Label ("$ 0.00", Label.RIGHT);
			totalPrice.setForeground (Color.blue);
			totalPrice.setFont (bigFont);


		addElement (NORTH, l_code, 1,1,1,1);
		addElement (NORTH, l_description, 2,1,1,1);
		addElement (NORTH, l_qty, 3,1,1,1);
		addElement (NORTH, l_price, 4,1,1,1);

		for (i = 0; i < 12; i++)
		{
			addElement (NORTH, code[i], 1,2+i,1,1);
			addElement (NORTH, description[i], 2,2+i,1,1);
			addElement (NORTH, qty[i], 3,2+i,1,1);
			addElement (NORTH, price[i], 4,2+i,1,1);
		}

		addElement (SOUTH, l_total, 2, 14, 2, 1);
		addElement (SOUTH, totalPrice, 4, 14, 1, 1);

		code[0].requestFocus ();
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_ACTION)
		{
			if ((evt.target instanceof InputField) == false)
				return super.handleEvent (evt);

			InputField k = (InputField) evt.target;
			int theRow = fieldRow (k);
			int fieldType = editFieldType (k);

			if (evt.key == Event.LEFT)
			{
				if (fieldType ==  CODE && theRow > 0)
					code[theRow-1].requestFocus();
				else if (fieldType == DESC)
					code[theRow].requestFocus();
				else if (fieldType == QTY)
					description[theRow].requestFocus();
			}
			if (evt.key == Event.RIGHT)
			{
				if (fieldType == CODE)
					description[theRow].requestFocus();
				else if (fieldType == DESC)
					qty[theRow].requestFocus(); 
				else if (fieldType == QTY && theRow < (12 - 1))
					code[theRow+1].requestFocus(); 
			}

			if (evt.key == Event.UP)
			{
				if (theRow > 0)
				{
					if (fieldType == CODE)
						code[theRow-1].requestFocus();
					else if (fieldType == DESC)
						description[theRow-1].requestFocus();
					else if (fieldType == QTY)
						qty[theRow-1].requestFocus();
				}
			}
			if (evt.key == Event.DOWN)
			{
				if (theRow < 11)
				{
					if (fieldType == CODE)
						code[theRow+1].requestFocus();
					else if (fieldType == DESC)
						description[theRow+1].requestFocus();
					else if (fieldType == QTY)
						qty[theRow+1].requestFocus();
				}
			}
		}
		return super.handleEvent (evt);
	}

	public void recalcPrice ()
	{
		double thePrice = 0.0;

		for (int i = 0; i < 12; i++)
		{
			thePrice += price[i].getDouble();
		}

		totalPrice.setText ("$ " + Money.toString(thePrice));
	}

	public int editFieldType (InputField c)
	{
		for (int i = 0; i < 12; i++)
		{
			if (code[i] == c)
				return CODE;
			else if (description[i] == c)
				return DESC;
			else if (qty[i] == c)
				return QTY;
			else if (price[i] == c)
				return PRICE;
		}
		return -1;
	}

	public int fieldRow (InputField c)
	{
		for (int i = 0; i < 12; i++)
		{
			if (code[i] == c)
				return i;
			else if (description[i] == c)
				return i;
			else if (qty[i] == c)
				return i;
			else if (price[i] == c)
				return i;
		}
		return -1;
	}
}

class OrderEntry extends GridFrame {

	Order myOrder;
	Server theServer;
	boolean newOrder;
	Button acceptBtn, cancelBtn, closeBtn;
	OrderMenuCanvas menuPic;
	OrderEntryPanel orderPanel;
	String promptString1, promptString2;
	InputField orderName, orderTime;
	Select theParent;
	Customer myCust;
	double oldPrice;
	int onRow;
	Color focusColor;
	String theName, theTime;

	Item myItem;

	int CODE = 1;
	int DESC = 2;
	int QTY  = 3;
	int PRICE = 4;

	public OrderEntry (Select ss, Customer c, Server s)
	{
		super ("New Order Details");
		theParent = ss;
		theName = null;
		theTime = null;
		theServer = s;
		myOrder = new Order ();
		myCust = c;
		if (myCust == null)
		{
			myOrder.isPickup = true;
		}
		else
		{
			promptString1 = "Customer: " + myCust.name + ", " + myCust.address;
			promptString2 = "Delivery to " + myCust.suburb + " UBD " + myCust.ubd + " Min Order " + myCust.minOrder;
			myOrder.custID = myCust.ID;
		}

		newOrder = true;
		myItem = null;
		initScreen ();
	}

	public OrderEntry (Select ss, Customer c, Server s, Order o)
	{
		super ("Old Order Details");
		theParent = ss;
		theServer = s;
		myOrder = o;
		myCust = c;
		if (myCust != null)
		{
			promptString1 = "Customer: " + myCust.name + ", " + myCust.address;
			promptString2 = "Delivery to " + myCust.suburb + " UBD " + myCust.ubd + " Min Order " + myCust.minOrder;
			myOrder.custID = myCust.ID;
		}

		oldPrice = myOrder.totalPrice();
		newOrder = false;
		myItem = null;
		initScreen ();
		orderToScreen ();
	}

	public void initScreen ()
	{
		Label l_prompt, l_prompt2;
		if (myOrder.isPickup)
		{
			l_prompt = new Label ("Pickup By:", Label.RIGHT);
			orderName = new InputField (30, InputField.TEXT);
			l_prompt2 = new Label ("Pickup Time:", Label.RIGHT);
			orderTime = new InputField (15, InputField.CAPS);
		}
		else
		{
			focusColor = new Color (130,252,237);
			l_prompt = new Label (promptString1, Label.CENTER);
			l_prompt.setForeground (new Color(0,0,64));
			l_prompt2 = new Label (promptString2, Label.CENTER);
			l_prompt2.setForeground (Color.blue);
		}

		orderPanel = new OrderEntryPanel ();

		menuPic = new OrderMenuCanvas ();

		acceptBtn = new Button ("Accept");
			acceptBtn.setBackground (new Color (145,200,162));

		if (newOrder == false)
		{
			cancelBtn = new Button ("Cancel Order");
				cancelBtn.setBackground (new Color (255,128,128));
		}

		closeBtn = new Button ("Close");
			closeBtn.setBackground (new Color (145,196,255));

		gridc.fill = GridBagConstraints.HORIZONTAL;
		gridc.ipadx = gridc.ipady = 0;
		gridc.weightx = gridc.weighty = 0.0;

		if (myOrder.isPickup)
		{
			addElement (NORTH, l_prompt, 1,1,1,1);
			addElement (NORTH, orderName, 2,1,1,1);
			addElement (NORTH, l_prompt2, 1,2,1,1);
			addElement (NORTH, orderTime, 2,2,1,1);
		}
		else
		{
			addElement (NORTH, l_prompt, 1,1,1,1);
			addElement (NORTH, l_prompt2, 1,2,1,1);
		}

		gridc.fill = 0;
		addElement (NORTHWEST, orderPanel, 1,3,2,1);

		addElement (NORTHEAST, menuPic, 3,1,3,3);

		gridc.ipadx = gridc.ipady = 20;
		addElement (SOUTHEAST, acceptBtn,   3,4,1,1);

		if (newOrder == false)
			addElement (SOUTHEAST, cancelBtn,   4,4,1,1);

		addElement (SOUTHEAST, closeBtn,   5,4,1,1);

		pack();
		show();
			
		if (myOrder.isPickup && orderName.getText().length() == 0)
		{
			orderName.requestFocus ();
		}
		else
			orderPanel.code[0].requestFocus();
	}

	void orderToScreen ()
	{
		if (myOrder == null) { System.out.println ("null order"); return; }
		if (myOrder.details == null) { System.out.println ("null det"); return; }

		if (myOrder.isPickup)
		{
			orderName.setText (myOrder.name);
			orderTime.setText (myOrder.time);
		}

		for (int i = 0; i < 12 && i < myOrder.details.size(); i++)
		{
			OrderDetail myDet = (OrderDetail) myOrder.details.elementAt (i);
			orderPanel.code[i].setText (myDet.code);
			orderPanel.description[i].setText (myDet.description);
			orderPanel.qty[i].setDouble (myDet.qty);
			orderPanel.price[i].setDouble (myDet.price);
		}
		orderPanel.recalcPrice();
		orderPanel.code[0].requestFocus();
	}

	void screenToOrder ()
	{
		if (newOrder && (myOrder.orderNum == 0))
		{
			myOrder.orderNum = theServer.newOrderNum();
		}
		if (myCust != null)
			myOrder.custID = myCust.ID;

		if (myOrder.isPickup)
		{
			myOrder.name = orderName.getText();
			myOrder.time = orderTime.getText();
		}

		myOrder.date = new Date();
		myOrder.details = new Vector ();

		for (int i = 0; i < 12; i++)
		{
			if (orderPanel.code[i].getText().length() > 0)
			{
				OrderDetail myDet = new OrderDetail ();
				myDet.code = orderPanel.code[i].getText();
				myDet.description = orderPanel.description[i].getText();
				myDet.qty = orderPanel.qty[i].getDouble();
				myDet.price = orderPanel.price[i].getDouble();
				myOrder.details.addElement (myDet);
			}
		}
	}

	int editFieldType (InputField c)
	{
		return orderPanel.editFieldType (c);
	}

	int fieldRow (InputField c)
	{
		return orderPanel.fieldRow (c);
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n')
		{
			int theField = editFieldType ((InputField) evt.target);
			int theRow = fieldRow ((InputField) evt.target);

			boolean OK = false;

			if (theRow != -1)
			{
			if (orderPanel.code[theRow].getText().length() == 0)
			{
				OK = true;
			}
			else if (orderPanel.description[theRow].getText().length() != 0)
			{
				OK = true;
			}

			if (OK)
			{
				myItem = theServer.getItem (orderPanel.code[theRow].getText());
				if (myItem != null)
				{
					double v = orderPanel.qty[theRow].getDouble();
					orderPanel.price[theRow].setDouble (myItem.price*v);
				}
				evt.id = Event.ACTION_EVENT;
				evt.target = acceptBtn;
				evt.arg = "Accept";
				super.deliverEvent (evt);
				return (true);
			}
			}
		}
		else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Close"))
		{
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.arg == "Cancel Order")
		{
			if (oldPrice > 0.0)
			{
				myOrder.details = new Vector (); // wipe out details
				if (myCust != null)
					myCust.totalSpent -= oldPrice;

				if (myOrder.isPickup)
				{
					myOrder.name = "** Cancelled ** " + myOrder.name;
				}
			}
			theServer.updateOrder (myOrder, oldPrice);
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.arg == "Accept")
		{
			// update the database
			screenToOrder ();
			double d = myOrder.totalPrice();

			if (newOrder)
			{
				if (d > 0.0)
				{
					if (myCust != null)
					{
						myCust.lastOrderNum = myOrder.orderNum;
						myCust.totalSpent += myOrder.totalPrice();
						myCust.lastOrderDate = myOrder.date;
					}
					theServer.newOrder (myOrder);
				}
			} else {
				if (myCust != null)
				{
					myCust.totalSpent -= oldPrice;
					myCust.totalSpent += myOrder.totalPrice();
				}
				theServer.updateOrder (myOrder, oldPrice);
			}
			// print the receipt
			myOrder.printDocket(myCust);
			dispose ();
			theParent.show();
			theParent.redoLabels();
		} else if (evt.id == Event.KEY_ACTION)
		{
			switch (evt.key)
			{
				case Event.F1: menuPic.setMenu (0); break;
				case Event.F2: menuPic.setMenu (1); break;
				case Event.F3: menuPic.setMenu (2); break;
				case Event.F4: menuPic.setMenu (3); break;
				case Event.F5: menuPic.setMenu (4); break;
				case Event.F6: menuPic.setMenu (5); break;
				case Event.F7: menuPic.setMenu (6); break;
				case Event.F8: menuPic.setMenu (7); break;
				case Event.F9: menuPic.setMenu (8); break;
				case Event.F10: menuPic.setMenu (9); break;
				case Event.F12:
					if (evt.target instanceof InputField)
					{
						new ItemSelectTable (theServer, this);
					}
					break;
			}
		} else if (evt.id == Event.GOT_FOCUS)
		{
			if (evt.target instanceof InputField)
			{
				((InputField)evt.target).setBackground (focusColor);
				int theField = editFieldType ((InputField) evt.target);
				int theRow = fieldRow ((InputField) evt.target);

				if (theRow != -1)
				{
					onRow = theRow;
		
					if (theField == CODE || theField == QTY)
					{
						InputField c = (InputField) evt.target;
						c.select(0,100);
					}
				}
			}
		} else if (evt.id == Event.LOST_FOCUS)
		{
			if (evt.target instanceof InputField)
			{
				((InputField)evt.target).setBackground (Color.white);
				int theField = editFieldType ((InputField) evt.target);
				int theRow = fieldRow ((InputField) evt.target);
		
				if (theRow != -1)
				{
					processFocusLost (theField, theRow);
					orderPanel.recalcPrice();
				}
			}
		}

		return (super.handleEvent (evt));
	}

	public void setItems (Vector myItems)
	{
		if (onRow != -1)
		{
			int i;
			for (i = 0; i < myItems.size(); i++)
			{
				if ((onRow + i) > 9) break;

				Item myItem = (Item) myItems.elementAt (i);
				orderPanel.code[onRow+i].setText (myItem.code);
				orderPanel.description[onRow+i].setText (myItem.description);
				double v = orderPanel.qty[onRow+i].getDouble();
				if (v == 0.0)
				{
					orderPanel.qty[onRow+i].setText ("1.0");
					orderPanel.price[onRow+i].setDouble (myItem.price);
				}
				else {
					orderPanel.price[onRow+i].setDouble (myItem.price*v);
				}
			}
			orderPanel.recalcPrice();
			i --;
			if ((onRow + i) > 9)
				orderPanel.code[9].requestFocus ();
			else
				orderPanel.code[onRow + i].requestFocus ();
		}
	}

	public void processFocusLost (int theField, int theRow)
	{
		if (theField != -1 && theRow != -1)
		{
			if (theField == QTY)
			{
				myItem = theServer.getItem (orderPanel.code[theRow].getText());
				if (myItem != null)
				{
					double v = orderPanel.qty[theRow].getDouble();
					orderPanel.price[theRow].setDouble (myItem.price*v);
				}
			}
			else if (theField == CODE)
			{
				myItem = theServer.getItem (orderPanel.code[theRow].getText());
				if (myItem != null)
				{ 
					if (orderPanel.description[theRow].getText().length() == 0)
						orderPanel.description[theRow].setText (myItem.description);
					double v = orderPanel.qty[theRow].getDouble();
					if (v == 0.0)
					{
						orderPanel.qty[theRow].setText ("1.0");
						orderPanel.price[theRow].setDouble (myItem.price);
					}
					else {
						orderPanel.price[theRow].setDouble (myItem.price*v);
					}
				}
				else {
					orderPanel.description[theRow].setText ("");
					orderPanel.qty[theRow].setText ("");
					orderPanel.price[theRow].setText ("");
				}
			}
		}
	}

}

