import java.awt.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Date;

class DataThread extends Thread {

	long expiresAt;
	Server theServer;
	boolean doSave;
	boolean doSaveOrders;

	public DataThread (Server s)
	{
		expiresAt = 0;
		theServer = s;
		doSave = false;
	}

	public void run ()
	{
		System.out.println ("Data thread started");
		try {
			while (true)
			{
				sleep (60000); // wait 1 minute between checks
				flush (false);
			}
		} catch (InterruptedException e)
		{
			e.printStackTrace ();
		}
	}

	public void flagSave (boolean includeOrders)
	{
		expiresAt = new Date().getTime() + 300000;	// 5 minutes idle time between saves, unless
													// more save messages come through
		if (includeOrders)
			doSaveOrders = true;
		doSave = true;
	}

	public void bumpTimer ()
	{
		// if we are already in a save state, then wait another 5 minutes, because 
		// we have just started doing something interesting

		expiresAt = new Date().getTime() + 300000;	// 10 minutes idle time between saves, unless
													// more save messages come through
	}

	public void flush (boolean forceFlush)
	{
		if (doSave)
		{
			long now = new Date().getTime ();
			if (forceFlush)
			{
				System.out.println ("Forced flush");
				doSave = false;
				theServer.hardStoreData (doSaveOrders);
				doSaveOrders = false;
			}
			else
			{
			//System.out.println ("Flushing, now = " + now + " alarm = " + expiresAt + " orders = " + doSaveOrders);
			if (now > expiresAt)
			{
				doSave = false;
				theServer.hardStoreData (doSaveOrders);
				doSaveOrders = false;
			}
			}
		}
		//else System.out.println ("Flushing, but nothing to save");
	}
}

class ProductStat extends Object {

	double qty;
	double price;

	public ProductStat (double q, double p)
	{
		qty = q;
		price = p;
	}
}

class Server {

	Stats theStats;
	Vector theUsers;
	Vector theCustomers;
	Vector theItems;
	Vector theSuburbs;
	Vector theOrders;
	DataThread myDataThread;
	int currentDay, currentMonth, currentYear;
	long currentStart, currentEnd;
	Frame mainFrame;

	public Server (Frame f)
	{
		mainFrame = f;
		theStats = new Stats();	

		Date myDate = new Date();
		if (myDate.getHours() < 9)
		{
			// is really the previous day !
			myDate = new Date (myDate.getTime() - 86400000);
		}

		currentDay = myDate.getDate();
		currentMonth = myDate.getMonth();
		currentYear = myDate.getYear();

		System.out.println ("d = " + currentDay + " m = " + currentMonth + " y = " + currentYear);
		Date d1 = new Date (currentYear, currentMonth, currentDay);
		currentStart = d1.getTime() + (9 * 3600000);
		currentEnd = currentStart + (24 * 3600000);

		loadData ();
		myDataThread = new DataThread (this);
		myDataThread.start ();
	}

	boolean sameDay (Date theDate)
	{
		long l = theDate.getTime();
		//System.out.println ("Comparing " + theDate + " to range " + new Date(currentStart) + ", " + new Date(currentEnd));

		if (l >= currentStart && l <= currentEnd)
			return true;
		else
			return false;
	}

	void loadData ()
	{
		int count,i;
		String str;

		try {
		DataInputStream fp = new DataInputStream (new FileInputStream ("pizza.data"));

		int Version = 0;
		long lastCust = 1000;
		Version = fp.readInt ();
		System.out.println ("Data version " + Version);

		// This is now redundant

		theStats.read (fp, Version);

		// Rebuild the stats on the fly from the orders data

		count = fp.readInt ();
		theUsers = new Vector (count,10);
		for (i = 0; i < count; i++)
		{
			User myUser = new User (fp, Version);
			theUsers.addElement (myUser);	
		}
		count = fp.readInt ();
		theCustomers = new Vector (count,20);
		boolean done384 = false;
		for (i = 0; i < count; i++)
		{
			Customer myCust = new Customer (fp, Version);
			if (myCust.ID == 0)
			{
				myCust.ID = lastCust;
				lastCust ++;
			}

			// patch to fix stuffed data    31-Jan-1998

			if (myCust.ID == 384)
			{
				done384 = true;
			}

			//if (myCust.ID >= 1000 && done384)
			//{
				//myCust.ID += 1500;
			//}

			theCustomers.addElement (myCust);	
		}
		count = fp.readInt ();
		theItems = new Vector (count,20);
		for (i = 0; i < count; i++)
		{
			Item myItem = new Item (fp, Version);
			theItems.addElement (myItem);	
		}
		count = fp.readInt ();
		theSuburbs = new Vector (count,20);
		for (i = 0; i < count; i++)
		{
			Suburb mySuburb = new Suburb (fp, Version);
			theSuburbs.addElement (mySuburb);
		}

		fp.close ();

		fp = new DataInputStream (new FileInputStream ("orders.data"));
		count = fp.readInt ();
		theOrders = new Vector (count,50);
		for (i = 0; i < count; i++)
		{
			Order myOrder = new Order (fp, Version);
			theOrders.addElement (myOrder);

			// Now add to stats
			if (sameDay (myOrder.date))
			{
				theStats.ordersToday ++;
				theStats.valueToday += myOrder.totalPrice();
			}
		}

		fp.close ();

		} catch (IOException e)
		{
			e.printStackTrace ();
		}
	}

	void storeData (boolean doOrders)
	{
		System.out.println ("save data message");
		myDataThread.flagSave (doOrders);
	}

	void bumpTimer ()
	{
		System.out.println ("bumped timer back to 5 minutes");
		myDataThread.bumpTimer ();
	}

	void flush ()
	{
		myDataThread.flush (true);
	}

	void hardStoreData (boolean doOrders)
	{
		System.out.println ("hard save data start");
		int count,i;

		try {
		DataOutputStream fp = new DataOutputStream (new FileOutputStream ("pizza.data"));

		// we are now on data version 2 - this includes the bad customer flag
		// we are now on data version 3 - this includes the customer discount

		int Version = 3;
		fp.writeInt (Version);

		theStats.write (fp);
		count = theUsers.size ();
		fp.writeInt (count);
		for (i = 0; i < count; i++)
		{
			User myUser = (User) theUsers.elementAt (i);
			myUser.write (fp);
		}
		count = theCustomers.size ();
		fp.writeInt (count);
		for (i = 0; i < count; i++)
		{
			Customer myCust = (Customer) theCustomers.elementAt (i);
			myCust.write (fp);
		}
		count = theItems.size ();
		fp.writeInt (count);
		for (i = 0; i < count; i++)
		{
			Item myItem = (Item) theItems.elementAt (i);
			myItem.write (fp);
		}
		count = theSuburbs.size ();
		fp.writeInt (count);
		for (i = 0; i < count; i++)
		{
			Suburb mySuburb = (Suburb) theSuburbs.elementAt (i);
			mySuburb.write (fp);
		}

		fp.close ();

		if (doOrders == false)
		{
			System.out.println ("hard save completed");
			return;
		}

		fp = new DataOutputStream (new FileOutputStream ("orders.data"));

		count = theOrders.size ();
		fp.writeInt (count);
		for (i = 0; i < count; i++)
		{
			Order myOrder = (Order) theOrders.elementAt (i);
			myOrder.write (fp);
		}

		fp.close ();

		} catch (IOException e)
		{
			e.printStackTrace ();
		}
		System.out.println ("hard save data completed - with orders");
	}

	public void importData ()
	{
		int count,i;
		String str;

		Vector tempUsers = new Vector(10,5);
		Vector tempCustomers = new Vector(300,20);
		Vector tempItems = new Vector(100,10);
		Vector tempSuburbs = new Vector(100,10);

		User myUser = null;
		Customer myCust = null;
		Item myItem = null;
		Suburb mySuburb = null;

		int MODE_STATS = 1;
		int MODE_USERS = 2;
		int MODE_CUSTOMERS = 3;
		int MODE_ITEMS = 4;
		int MODE_SUBURBS = 5;

		try {
		DataInputStream fp = new DataInputStream (new FileInputStream ("/fd/pizza.txt"));

		String Buffer;
		int mode = 0;

		while (true)
		{
			try {
			Buffer = fp.readLine ();
			if (Buffer == null)
				break;

			if (Buffer.startsWith (":"))
			{
				System.out.println ("Going into mode " + Buffer.substring (1));
				if (Buffer.startsWith (":STATS"))
					mode = MODE_STATS;
				else if (Buffer.startsWith (":USERS"))
					mode = MODE_USERS;
				else if (Buffer.startsWith (":CUSTOMERS"))
					mode = MODE_CUSTOMERS;
				else if (Buffer.startsWith (":ITEMS"))
					mode = MODE_ITEMS;
				else if (Buffer.startsWith (":SUBURBS"))
					mode = MODE_SUBURBS;

				Buffer = fp.readLine (); // burn a line
				if (Buffer == null) break;
				if (Buffer.length() > 0)
				{
					Buffer = fp.readLine (); // position onto 1st real line
					if (Buffer == null) break;
				}
			}

			if (Buffer.length() > 0)
			{
				System.out.println ("'" + Buffer + "'");
				switch (mode)
				{
					case 1:
						theStats.importString (Buffer);
						break;
					case 2:
						myUser = new User ();
						myUser.importString (Buffer);
						tempUsers.addElement (myUser);
						break;

					case 3:
						myCust = new Customer ();
						myCust.importString (Buffer);
						tempCustomers.addElement (myCust);
						break;

					case 4:
						myItem = new Item ();
						myItem.importString (Buffer);
						tempItems.addElement (myItem);
						break;

					case 5:
						mySuburb = new Suburb ();
						mySuburb.importString (Buffer);
						tempSuburbs.addElement (mySuburb);
						break;
				}
			}
			else
				mode = 0;


			} catch (EOFException e1)
			{
				System.out.println ("EOF");
				break;
			}

		}

		fp.close ();

		// now validate the array of new data

		// now replace the old with the new
		theUsers =  tempUsers;
		theCustomers =  tempCustomers;
		theItems =  tempItems;
		theSuburbs =  tempSuburbs;
		storeData (false);

		} catch (IOException e)
		{
			e.printStackTrace ();
		}
	}

	public void exportData ()
	{
		int count,i;

		try {
		PrintStream fp = new PrintStream (new FileOutputStream ("/fd/pizza.txt"));

		fp.println (":STATS");
		fp.println ("Orders Today\tValue Today");
		theStats.print (fp);
		fp.println ("");

		fp.println (":USERS");
		count = theUsers.size ();
		for (i = 0; i < count; i++)
		{
			User myUser = (User) theUsers.elementAt (i);
			if (i == 0)
				myUser.printHeader (fp);
			myUser.print (fp);
		}
		fp.println ("");

		fp.println (":CUSTOMERS");
		count = theCustomers.size ();
		for (i = 0; i < count; i++)
		{
			Customer myCust = (Customer) theCustomers.elementAt (i);
			if (i == 0)
				myCust.printHeader (fp);
			myCust.print (fp);
		}
		fp.println ("");

		fp.println (":ITEMS");
		count = theItems.size ();
		for (i = 0; i < count; i++)
		{
			Item myItem = (Item) theItems.elementAt (i);
			if (i == 0)
				myItem.printHeader (fp);
			myItem.print (fp);
		}
		fp.println ("");

		fp.println (":SUBURBS");
		count = theSuburbs.size ();
		for (i = 0; i < count; i++)
		{
			Suburb mySuburb = (Suburb) theSuburbs.elementAt (i);
			if (i == 0)
				mySuburb.printHeader (fp);
			mySuburb.print (fp);
		}

		fp.close ();

		} catch (IOException e)
		{
			e.printStackTrace ();
		}
	}

	public void doProdReport (Date dateFrom, Date dateTo)
	{
		try {
			PrintStream fp = new PrintStream (new FileOutputStream ("prodReport"));

			           //----------------------------------------

			String d1 = dateFrom.toString();
			String df1 = d1.substring (0, 11) + d1.substring (24);
			String d2 = dateTo.toString();
			String dt2 = d2.substring (0, 11) + d2.substring (24);

			fp.println ("       Product Report");
			fp.println (" Date From: " + df1);
			fp.println (" Date To  : " + dt2 + "\n");
			fp.println ("Product      Qty     %    Price      % ");
			fp.println ("=======================================");
					   //oooooo 1234567890123456789 nnnnn nnn.nn

			double totalQty = 0.0;
			double totalPrice = 0.0;

			double catQty = 0.0;
			double catPrice = 0.0;

			long tStart, tEnd, tThis;
			long Hours9 = 32400000;
			long Hours24 = 86400000;
			tStart = dateFrom.getTime() + Hours9;
			tEnd = dateTo.getTime() + Hours24 + Hours9;

			Date x1 = new Date (tStart);
			Date x2 = new Date (tEnd);
			System.out.println ("From " + dateFrom + " -> " + x1 + " to " + dateTo + " -> " + x2);
			// construct a vector of products
			Hashtable ProdStats = new Hashtable ();

			for (int i = 0; i < theOrders.size(); i++)
			{
				Order myOrder = (Order) theOrders.elementAt (i);
				tThis = myOrder.date.getTime();
				if (tThis >= tStart && tThis < tEnd)
				{
					for (int j = 0; j < myOrder.details.size(); j++)
					{
						OrderDetail od = (OrderDetail) myOrder.details.elementAt (j);
						ProductStat ps = (ProductStat) ProdStats.get (od.code);
						if (ps == null)
						{
							//System.out.println (od.code + " is not there");
							ps = new ProductStat (od.qty, od.price);
							ProdStats.put (od.code, ps);
							//System.out.println ("  Size is now " + ProdStats.size());
						}
						else
						{
							//System.out.println (od.code + " is there");
							ps.qty += od.qty;
							ps.price += od.price;
							//System.out.println ("  Stats is now " + ps.qty + " " + ps.price);
						}
						totalQty += od.qty;
						totalPrice += od.price;
					}
				}
			}

			// now print the report
			String lastType = " ";
			catQty = 0.0;
			catPrice = 0.0;
			Item myItem;
			ProductStat ps;
			String m;
			double thePercent;
			for (int i = 0; i < theItems.size(); i++)
			{
				myItem = (Item) theItems.elementAt (i);
				ps = (ProductStat) ProdStats.get (myItem.code);
				//System.out.println ("Looking for stats on " + myItem.code + " got " + ps);
				if (ps != null)
				{
					String s = myItem.category + " " + myItem.code;
					while (s.length() < 10) s = s + " ";
					if (s.length() > 10) s = s.substring (0,10);

					// add the qty 
					m = "" + ps.qty;
					while (m.length() < 4) m = " " + m;
					s = s + " " + m;

					// add the qty as a %
					thePercent = ps.qty / totalQty * 100.0;
					m = Money.toString (thePercent);
					while (m.length() < 6) m = " " + m;
					s = s + " " + m;

					// add the price
					m = Money.toString (ps.price);
					while (m.length() < 8) m = " " + m;
					s = s + " " + m;

					// add the price as a %
					thePercent = ps.price / totalPrice * 100.0;
					m = Money.toString (thePercent);
					while (m.length() < 6) m = " " + m;
					s = s + " " + m;

					if (myItem.category.equals(lastType) == false)
					{
						if (lastType.equals (" ") == false)
						{
							// print totals for this group
							fp.println ("---------------------------------------");
							//	 		 Product      Qty     %    Price      % 
							//	 		 =======================================
							//			 P SVG         1   0.49     9.00   0.43
							//			 P SSG         1   0.49     9.50   0.45
							String s1 = "          ";

							// add the qty 
							m = "" + catQty;
							while (m.length() < 4) m = " " + m;
							s1 = s1 + " " + m;

							// add the qty as a %
							thePercent = catQty / totalQty * 100.0;
							m = Money.toString (thePercent);
							while (m.length() < 6) m = " " + m;
							s1 = s1 + " " + m;

							// add the price
							m = Money.toString (catPrice);
							while (m.length() < 8) m = " " + m;
							s1 = s1 + " " + m;

							// add the price as a %
							thePercent = catPrice / totalPrice * 100.0;
							m = Money.toString (thePercent);
							while (m.length() < 6) m = " " + m;
							s1 = s1 + " " + m;

							fp.println (s1);
							fp.println (" ");
							fp.println ("---------------------------------------");
							catQty = ps.qty;
							catPrice = ps.price;
						}
						else // very first one
						{
							catQty = ps.qty;
							catPrice = ps.price;
						}
						lastType = myItem.category;
					}
					else // add to category stats
					{
						catQty += ps.qty;
						catPrice += ps.price;
					}

					fp.println (s);
				}
			}
			String s1 = "          ";

			// add the qty 
			m = "" + catQty;
			while (m.length() < 4) m = " " + m;
			s1 = s1 + " " + m;

			// add the qty as a %
			thePercent = catQty / totalQty * 100.0;
			m = Money.toString (thePercent);
			while (m.length() < 6) m = " " + m;
			s1 = s1 + " " + m;

			// add the price
			m = Money.toString (catPrice);
			while (m.length() < 8) m = " " + m;
			s1 = s1 + " " + m;

			// add the price as a %
			thePercent = catPrice / totalPrice * 100.0;
			m = Money.toString (thePercent);
			while (m.length() < 6) m = " " + m;
			s1 = s1 + " " + m;

			fp.println ("---------------------------------------");
			fp.println (s1);

			fp.println ("=======================================");
			String ttl = "  Total Qty: ";

					m = "" + totalQty;
					while (m.length() < 6) m = "  " + m;
					ttl = ttl + " " + m;
			fp.println (ttl);

				   ttl = "  Total Price: ";
					m = Money.toString (totalPrice);
					while (m.length() < 9) m = "  " + m;
					ttl = ttl + " " + m;

			fp.println (ttl);
			fp.println ("=======================================");
			fp.println ("\n\n\n\n\n\n");

			fp.flush ();
			fp.close ();

		} catch (Exception e)
		{
			e.printStackTrace ();
		}
	}

	public void doReport (Date dateFrom, Date dateTo)
	{
		try {
			PrintStream fp = new PrintStream (new FileOutputStream ("report"));

			           //----------------------------------------

			String d1 = dateFrom.toString();
			String df1 = d1.substring (0, 11) + d1.substring (24);
			String d2 = dateTo.toString();
			String dt2 = d2.substring (0, 11) + d2.substring (24);

			fp.println ("       Sales Report");
			fp.println (" Date From: " + df1);
			fp.println (" Date To  : " + dt2 + "\n");
			fp.println ("Order   Phone/Name          Time  Price");
			fp.println ("=======================================");
					   //oooooo 1234567890123456789 nnnnn nnn.nn

			double totalQty = 0.0;
			double totalPrice = 0.0;

			double thisQty = 0.0;
			double thisPrice = 0.0;

			long tStart, tEnd, tThis;
			long Hours9 = 32400000;
			long Hours24 = 86400000;
			tStart = dateFrom.getTime() + Hours9;
			tEnd = dateTo.getTime() + Hours24 + Hours9;

			Date x1 = new Date (tStart);
			Date x2 = new Date (tEnd);
			System.out.println ("From " + dateFrom + " -> " + x1 + " to " + dateTo + " -> " + x2);

			for (int i = 0; i < theOrders.size(); i++)
			{
				Order myOrder = (Order) theOrders.elementAt (i);
				tThis = myOrder.date.getTime();
				if (tThis >= tStart && tThis < tEnd)
				{
					thisQty = myOrder.details.size();
					thisPrice = myOrder.totalPrice();
					totalQty += thisQty;
					totalPrice += thisPrice;

					String s = "" + myOrder.orderNum;

					if (myOrder.isPickup)
					{
						s = s + " P";
						while (s.length() < 8)
							s = s + " ";
						s = s + myOrder.name;
					}
					else
					{
					while (s.length() < 8)
						s = s + " ";
						Customer myCust = findCustomerID (myOrder.custID);
						s = s + myCust.phone;
					}

					while (s.length() < 26) s = s + " ";
					if (s.length() > 26) s = s.substring (0, 26);

					s = s + " " + myOrder.date.toString().substring (11,16);

					String m = Money.toString (thisPrice);
					while (m.length() < 6) m = " " + m;
					s = s + " " + m;

					fp.println (s);
				}
			}

			fp.println ("=======================================");
			String ttl = "                        Total: ";

					String m = Money.toString (totalPrice);
					while (m.length() < 6) m = "  " + m;
					ttl = ttl + " " + m;

			fp.println (ttl);
			fp.println ("=======================================");
			fp.println ("\n\n\n\n\n\n");

			fp.flush ();
			fp.close ();

		} catch (Exception e)
		{
			e.printStackTrace ();
		}
	}

	public void badCustReport ()
	{
		try {
			PrintStream fp = new PrintStream (new FileOutputStream ("badCust"));

			           //----------------------------------------

			fp.println ("   Bad Customer Report\n");
			Date d = new Date();
			String s = InputField.DateString (d);
			fp.println (" As of Date : " + s + "\n");


			for (int i = 0; i < theCustomers.size(); i++)
			{
				Customer myCust = (Customer) theCustomers.elementAt(i);
				if (myCust.badCust)
				{
					fp.println ("=========================================");
					fp.println ("Name   : " + myCust.name);
					fp.println ("Addr   : " + myCust.address);
					fp.println ("Phone  : " + myCust.phone);
					fp.println ("Suburb : " + myCust.suburb);
					fp.println ("UBD    : " + myCust.ubd);
					if (myCust.comments.length() > 0)
					{
						fp.println ("");
						fp.println ("Comments :");
						fp.println (myCust.comments);
					}
					fp.println ("");
					fp.println ("Last Order  : " + myCust.lastOrderNum);
					fp.println ("            : " + InputField.DateString (myCust.lastOrderDate));
					fp.println ("");
					fp.println ("Total Spent : " + Money.toString(myCust.totalSpent));

					if (myCust.minOrder != null)
						fp.println ("Min Order   : " + myCust.minOrder);
				}
			}

			fp.println ("\n\n\n\n\n\n");

			fp.flush ();
			fp.close ();

		} catch (Exception e)
		{
			e.printStackTrace ();
		}
	}

	public void printCustReport (Customer myCust)
	{
		try {
			PrintStream fp = new PrintStream (new FileOutputStream ("custReport"));

			           //----------------------------------------

			fp.println ("       Customer Report\n");
			Date d = new Date();
			String s = InputField.DateString (d);
			fp.println (" As of Date : " + s);
			fp.println ("\nName  : " + myCust.name);
			fp.println ("Addr  : " + myCust.address);
			fp.println ("        " + myCust.suburb);
			fp.println ("Phone : " + myCust.phone);
			fp.println ("");
			if (myCust.comments.length() > 0)
			{
				fp.println ("Comments :");
				fp.println (myCust.comments);
				fp.println ("");
			}
			fp.println ("Order Date                  Items   Price");
			fp.println ("=========================================");
					   //ooooo 123456789012345678901  nnnn nnnn.nn

			double totalQty = 0.0;
			double totalPrice = 0.0;

			double thisQty = 0.0;
			double thisPrice = 0.0;

			Vector custOrders = getOrdersByCust (myCust);

			for (int i = custOrders.size() - 1; i >= 0; i--)
			{
				Order myOrder = (Order) custOrders.elementAt (i);
				thisQty = myOrder.totalQty();
				thisPrice = myOrder.totalPrice();
				totalQty += thisQty;
				totalPrice += thisPrice;

				s = "" + myOrder.orderNum;

				while (s.length() < 6) s = s + " ";

				s = s + InputField.DateString (myOrder.date);

				while (s.length() < 27) s = s + " ";
					if (s.length() > 27) s = s.substring (0, 27);

				String m = "" + thisQty;
				while (m.length() < 5) m = " " + m;
				s = s + " " + m;

				m = Money.toString (thisPrice);
				while (m.length() < 7) m = " " + m;
				s = s + " " + m;

				fp.println (s);
			}

			custOrders = null;

			fp.println ("=========================================");
			String ttl = "                        Total: ";

					String m = Money.toString (totalPrice);
					while (m.length() < 8) m = "  " + m;
					ttl = ttl + " " + m;

			fp.println (ttl);
			fp.println ("=========================================");
			fp.println ("\n\n\n\n\n\n");

			fp.flush ();
			fp.close ();

		} catch (Exception e)
		{
			e.printStackTrace ();
		}
	}
	public boolean validateLogon (String name, String password)
	{
		for (int i =0; i < theUsers.size(); i++)
		{
			User myUser = (User) theUsers.elementAt (i);
			if (myUser.name.equals (name) &&
			    myUser.password.equals (password))
			    	return true;
		}
		return false;
	}

	public Customer findCustomerID (long ID)
	{
		for (int i = 0; i < theCustomers.size (); i++)
		{
			Customer myCust = (Customer) theCustomers.elementAt (i);

			if (myCust.ID == ID)
				return myCust;
		}
		return null;
	}

	public Customer findCustomer (String name)
	{
		for (int i = 0; i < theCustomers.size (); i++)
		{
			Customer myCust = (Customer) theCustomers.elementAt (i);

			if (myCust.name.equals(name))
				return myCust;
		}
		return null;
	}

	public Vector findCustomers (String phone, String name)
	{
		Vector retVal = new Vector ();

		for (int i = 0; i < theCustomers.size (); i++)
		{
			Customer myCust = (Customer) theCustomers.elementAt (i);

			if ((myCust.name.indexOf (name) != -1) &&
			    (myCust.phone.indexOf (phone) != -1))
			    {
			    	retVal.addElement (myCust);
			    }
		}

		if (retVal.size() == 0)
			retVal = null;

		return retVal;
	}


	public Vector getOrdersByCust (Customer theCust)
	{
		Vector retVal = new Vector ();
		for (int i = 0; i < theOrders.size(); i++)
		{
			Order myOrder = (Order) theOrders.elementAt (i);
			if (myOrder.custID == theCust.ID)
				retVal.addElement (myOrder);
		}
		return retVal;
	}

	public Customer findCustByOrder (int order)
	{
		for (int i =0; i < theOrders.size(); i++)
		{
			Order myOrder = (Order) theOrders.elementAt (i);
			if (myOrder.orderNum == order)
			{
				for (int j = 0; j < theCustomers.size(); j++)
				{
					Customer myCust = (Customer) theCustomers.elementAt (j);
					if (myCust.name.equals (myOrder.name))
						return myCust;
				}
				return null;
			}
		}
		return null;
	}

	public Item getItem (String code)
	{
		for (int i = 0; i < theItems.size(); i++)
		{
			Item myItem = (Item) theItems.elementAt (i);
			if (myItem.code.equals (code))
				return myItem;
		}
		return null;
	}

	public void updateCust (Customer theCust)
	{
		storeData (false);
	}

	public boolean custExists (String c)
	{
		for (int i = 0; i < theCustomers.size(); i++)
		{
			Customer myCust = (Customer) theCustomers.elementAt (i);
			if (myCust.name.equals (c))
				return true;
		}
		return false;
	}

	public boolean custExistsPhone (String c)
	{
		for (int i = 0; i < theCustomers.size(); i++)
		{
			Customer myCust = (Customer) theCustomers.elementAt (i);
			if (myCust.phone.equals (c))
				return true;
		}
		return false;
	}

	public boolean newCust (Customer theCust)
	{
		if (theCust.name.length() > 0)
		{
			if (custExists (theCust.name))
			{
				new MessageBox (mainFrame, "Error", "There is already a customer with this name");
				return false;
			}
			else
			{
				theCust.ID = theCustomers.size() + 3000;
				theCustomers.addElement (theCust);
				storeData (false);
				return true;
			}
		}
		else if (theCust.phone.length() > 0)
		{
			if (custExistsPhone (theCust.phone))
			{
				new MessageBox (mainFrame, "Error", "There is already a customer with this phone number");
				return false;
			}
			else
			{
				theCust.ID = theCustomers.size() + 3000;
				theCustomers.addElement (theCust);
				storeData (false);
				return true;
			}
		}
		else new MessageBox (mainFrame, "Error", "Cannot enter customer with blank name and phone number");
		return false;

	}

	public void deleteCust (Customer theCust)
	{
		theCustomers.removeElement (theCust);
		storeData (false);
	}

	public int newOrderNum ()
	{
		return (theOrders.size() + 1);
	}

	public Vector getLastOrders (int count)
	{
		Vector result = new Vector ();
		int x = theOrders.size() - 1;
		for (int i = 0; i < count && x >= 0; i++)
		{
			Order myOrder = (Order) theOrders.elementAt (x);
			x--;
			result.addElement (myOrder);
		}
		return result;
	}

	public void newOrder (Order myOrder)
	{
		theOrders.addElement (myOrder);
		theStats.ordersToday ++;
		theStats.valueToday += myOrder.totalPrice();
		storeData (true);
	}

	public void updateOrder (Order myOrder, double oldPrice)
	{
		theStats.valueToday -= oldPrice;
		theStats.valueToday += myOrder.totalPrice();
		storeData (true);
	}

	public Order getOrder (long orderNum)
	{
		for (int i = 0; i < theOrders.size(); i++)
		{
			Order myOrder = (Order) theOrders.elementAt (i);
			if (myOrder.orderNum == orderNum)
				return myOrder;
		}
		return null;
	}

	public void deleteOrder (int order)
	{
	}

	public void printOrder (int order)
	{
	}

	public boolean userExists (String u)
	{
		for (int i = 0; i < theUsers.size(); i++)
		{
			User myUser = (User) theUsers.elementAt (i);
			if (myUser.name.equals (u))
				return true;
		}
		return false;
	}

	public void newUser (User theUser)
	{
		if (theUser.name.length() > 0)
		{
			if (userExists(theUser.name))
			{
				new MessageBox (mainFrame, "Error", "This user name is already used");
			}
			else
			{
				theUsers.addElement (theUser);
				storeData (false);
			}
		}
		else new MessageBox (mainFrame, "Error", "Cannot enter blank user code");
	}

	public void updateUser (User theUser)
	{
		storeData (false);
	}

	public void deleteUser (User theUser)
	{
		theUsers.removeElement (theUser);
		storeData (false);
	}

	public boolean itemExists (String itm)
	{
		for (int i = 0; i < theItems.size(); i++)
		{
			Item myItem = (Item) theItems.elementAt (i);
			if (myItem.code.equals (itm))
				return true;
		}
		return false;
	}

	public void newItem (Item itm)
	{
		if (itm.code.length() > 0)
		{
			if (itemExists(itm.code))
			{
				new MessageBox (mainFrame, "Error", "This item code is already used");
			}
			else
			{
				theItems.addElement (itm);
				storeData (false);
			}
		} else new MessageBox (mainFrame, "Error", "Cannot add blank item code");
	}

	public void updateItem (Item itm)
	{
		storeData (false);
	}

	public void deleteItem (Item itm)
	{
		theItems.removeElement (itm);
		storeData (false);
	}

	public boolean suburbExists (String sub)
	{
		for (int i = 0; i < theSuburbs.size(); i++)
		{
			Suburb mySuburb = (Suburb) theSuburbs.elementAt (i);
			if (mySuburb.suburb.equals (sub))
				return true;
		}
		return false;
	}

	public Suburb matchSuburb (String sub)
	{
		System.out.println ("Looking for match on '" + sub + "'");
		Suburb retVal = null;
		int count = 0;
		if (sub != null && sub.length() > 0)
		{
			for (int i = 0; i < theSuburbs.size(); i++)
			{
				Suburb mySuburb = (Suburb) theSuburbs.elementAt (i);
				if (mySuburb.suburb.equals (sub))
				{
					// exact match - return this
					return mySuburb;

					// in the case where we have
					//  HAPPYVILLE
					//  HAPPYVILLE HEIGHTS
					//  HAPPYVILLE PARK
					//
					// Looking for a match on 'HAPPYVILLE' should return the
					// one with the exact match.
					//
					// Looking for a match on 'HAPPY' should return nothing yet
				}
				if (mySuburb.suburb.startsWith (sub))
				{
					System.out.println ("Found '" + mySuburb.suburb + "'");
					retVal = mySuburb;
					count++;
				}
			}
		}
		System.out.println ("Count = " + count);
		if (count == 1)
			return retVal;
		return null;
	}

	public void newSuburb (Suburb itm)
	{
		if (itm.suburb.length() > 0)
		{
			if (suburbExists (itm.suburb))
			{
				new MessageBox (mainFrame, "Error", "This suburb has already been entered");
			}
			else
			{
				theSuburbs.addElement (itm);
				storeData (false);
			}
		}
		else new MessageBox (mainFrame, "Error", "Cannot add a blank suburb");
	}

	public void updateSuburb (Suburb itm)
	{
		storeData (false);
	}

	public Suburb getSuburb (String s)
	{
		for (int i = 0; i < theSuburbs.size(); i++)
		{
			Suburb mySub = (Suburb) theSuburbs.elementAt (i);
			if (mySub.suburb.equals (s))
			{
				return mySub;
			}
		}
		return null;
	}

	public void setMinOrder (String suburb, double minOrder)
	{
		Suburb s = getSuburb (suburb);
		if (s != null)
		{
			s.minOrder = minOrder;
			storeData (false);
		}
	}

	public void deleteSuburb (Suburb itm)
	{
		theSuburbs.removeElement (itm);
		storeData (false);
	}

	public void check_orders (boolean fix)
	{
		try {
			PrintStream fp = new PrintStream (new FileOutputStream ("orders.check"));

			fp.println ("There are " + theOrders.size() + " orders to check");
			int count = 0;
			for (int i = 0; i < theOrders.size(); i++)
			{
				count = 1;
				Order theOrder = (Order) theOrders.elementAt (i);
				long checkOrder = theOrder.orderNum;
				fp.println ("element " + i + " = " + theOrder.orderNum);

				if (checkOrder != (i+1))
				{
					fp.println (" order number should be " + (i+1));
					if (fix)
					{
						fp.println (">>>>>>> Fixing ..");
						theOrder.orderNum = i+1;
						checkOrder = i+1;
					}
				}

				for (int j = 0; j < i; j++)
				{
					if (j != i)
					{
						Order thisOrder = (Order) theOrders.elementAt (j);
						if (thisOrder.orderNum == checkOrder)
						{
							count++;
							fp.println ("   Element " + j + " has same ordernum !");
						}
					}
				}
				if (count > 1)
				{
					fp.println ("   Order " + checkOrder + " is used " + count + " times !");
				}
			}
			fp.println ("Check complete");
		} catch (IOException ex)
		{
			ex.printStackTrace ();
		}
	}

	public void check_cust (boolean fix)
	{
		try {
			PrintStream fp = new PrintStream (new FileOutputStream ("cust.check"));

			fp.println ("There are " + theCustomers.size() + " Customers to check");
			int count = 0;
			for (int i = 0; i < theCustomers.size(); i++)
			{
				count = 1;
				Customer theCust = (Customer) theCustomers.elementAt (i);
				long checkCust = theCust.ID;
				fp.println ("element " + i + " = " + theCust.ID);

				if (checkCust != (i+1))
				{
					fp.println (" ID should be " + (i+1));
					if (fix)
					{
						fp.println (">>>>> Fixing ...");
						theCust.ID = i+1;

						// now hack the orders if there are no other customers
						// that mistakenly have this same order number as the old one

						boolean doIt = true;

						for (int j = i+1; j < theCustomers.size(); j++)
						{
							if (j != i)
							{
								Customer thisCust = (Customer) theCustomers.elementAt (j);
								if (thisCust.ID == checkCust)
								{
									doIt = false;
									fp.println ("  But there is another customer in the future with this ID");			
									break;
								}
							}
						}

						if (doIt)
						{
							for (int o = 0; o < theOrders.size(); o++)
							{
								Order theOrd = (Order) theOrders.elementAt (o);
								if (theOrd.custID == checkCust)
								{
									theOrd.custID = theCust.ID;
									fp.println ("   - Order " + theOrd.orderNum + " changed custID");
								}
							}
						}

						checkCust = i+1;
					}
				}

				for (int j = 0; j < i; j++)
				{
					if (j != i)
					{
						Customer thisCust = (Customer) theCustomers.elementAt (j);
						if (thisCust.ID == checkCust)
						{
							count++;
							fp.println ("   Elements " + i + " and " + j + 
								" has same cust ID of " + checkCust);
							fp.println ("   Address: " + thisCust.address + " " + thisCust.phone);
						}
					}
				}
				if (count > 1)
				{
					fp.println ("   Customer " + checkCust + " is used " + count + " times !");
				}
			}

			if (fix)
			{
				fp.println ("Resetting last order num & last order date");

				for (int i = 0; i < theCustomers.size(); i++)
				{
					Customer theCust = (Customer) theCustomers.elementAt (i);
					for (int j = theOrders.size() - 1; j >= 0; j--)
					{
						Order theOrd = (Order) theOrders.elementAt (j);
						if (theOrd.custID == theCust.ID)
						{
							fp.println ("Cust " + theCust.ID + " last order " + theOrd.orderNum + " on " + theOrd.date);
							theCust.lastOrderNum = theOrd.orderNum;
							theCust.lastOrderDate = theOrd.date;
							break;
						}
					}
				}
			}
			fp.println ("Check complete");
		} catch (IOException ex)
		{
			ex.printStackTrace ();
		}
	}
}

