import java.io.*;
import java.awt.*;

class Suburb extends Object {

	String suburb;
	int postcode;
	double minOrder;
	String ubd;

	public Suburb ()
	{
		suburb = null;
		postcode = 0;
		minOrder = 0.0;
		ubd = null;
	}

	public Suburb (DataInputStream fp, int Version) throws IOException
	{
		read (fp, Version);
	}

	public void read (DataInputStream in, int Version) throws IOException
	{
		suburb = in.readUTF();
		postcode = in.readInt();
		minOrder = in.readDouble();
		ubd = in.readUTF();	
	}

	public void write (DataOutputStream out) throws IOException
	{
		out.writeUTF(suburb);
		out.writeInt(postcode);
		out.writeDouble(minOrder);
		out.writeUTF (ubd);
	}

	public void print (PrintStream out) throws IOException
	{
		String t = "\t";
		out.println (suburb + t + postcode + t + minOrder + t + ubd);
	}

	public void printHeader (PrintStream out) throws IOException
	{
		out.println ("Suburb\tPostcode\tMin Order\tUBD");
	}

	public void importString (String s)
	{
		String t = "\t";
		int i;

		i = s.indexOf (t);
		suburb = s.substring (0, i); s = s.substring (i+1); i = s.indexOf (t);
		postcode = Integer.parseInt (s.substring (0, i)); s = s.substring (i+1); i = s.indexOf (t);
		minOrder = new Double(s.substring (0, i)).doubleValue(); s = s.substring (i+1); i = s.indexOf (t);
		if (i != -1) ubd = s.substring (0, i);
		else ubd = new String (" ");
	}
}

class SuburbEdit extends GridFrame {

	InputField suburb, postcode, ubdCode, minOrder;
	Button acceptBtn;
	Button cancelBtn;
	Server theServer;
	boolean newSuburb;
	Suburb oldSuburb;
	SuburbTable parentList;
	CreateMenu CM;

	public SuburbEdit (CreateMenu c, Server s, SuburbTable l)
	{
		super ("New Suburb");
		CM = c;
		CM.addWindow ("SuburbEdit", (GridFrame) this);
		newSuburb = true;
		oldSuburb = null;
		theServer = s;
		initScreen ();
		parentList = l;
	}

	public SuburbEdit (CreateMenu c, Server s, SuburbTable l, Suburb os)
	{
		super ("Modify Suburb Details");
		CM = c;
		CM.addWindow ("SuburbEdit", (GridFrame) this);
		newSuburb = false;
		oldSuburb = os;
		theServer = s;
		initScreen ();
		parentList = l;
	}

	public void initScreen ()
	{
		Label l_suburb = new Label ("Suburb:", Label.RIGHT);
		suburb = new InputField (30, InputField.CAPS);

		Label l_postcode = new Label ("Postcode:", Label.RIGHT);
		postcode = new InputField (8, InputField.INT);

		Label l_ubd = new Label ("UBD Code:", Label.RIGHT);
		ubdCode = new InputField (5, InputField.CAPS);

		Label l_minOrder = new Label ("Minumum Order:", Label.RIGHT);
		minOrder = new InputField (10, InputField.MONEY);

		if (newSuburb == false)
		{
			suburb.setText (oldSuburb.suburb);
			postcode.setInt (oldSuburb.postcode);
			ubdCode.setText (oldSuburb.ubd);
			minOrder.setDouble (oldSuburb.minOrder);
		}

		acceptBtn = new Button ("Accept");
			acceptBtn.setBackground (new Color (143, 200, 162));

		cancelBtn = new Button ("Cancel");
			cancelBtn.setBackground (new Color (255,128,128));


		addElement (EAST, l_suburb, 1, 1, 1, 1);
		addElement (WEST, suburb,   2, 1, 0, 1);
		addElement (EAST, l_postcode,   1, 2, 1, 1);
		addElement (WEST, postcode,   2, 2, 0, 1);
		addElement (EAST, l_ubd,   1, 3, 1, 1);
		addElement (WEST, ubdCode,   2, 3, 0, 1);
		addElement (EAST, l_minOrder,   1, 4, 1, 1);
		addElement (WEST, minOrder,   2, 4, 0, 1);
		addElement (SOUTHEAST, acceptBtn,  2, 5, 1, 1);
		addElement (SOUTHEAST, cancelBtn,  4, 5, 1, 1);
		pack ();

		show ();
		suburb.requestFocus ();
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n')
		{
			evt.id = Event.ACTION_EVENT;
			evt.target = acceptBtn;
			evt.arg = "Accept";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			CM.removeWindow ((GridFrame) this);
			dispose();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Cancel"))
		{
			CM.removeWindow ((GridFrame) this);
			dispose ();
		} else if (evt.arg == "Accept")
		{
			Suburb mySuburb;
			if (newSuburb)
				mySuburb = new Suburb ();
			else
				mySuburb = oldSuburb;

			boolean doIt = true;
			if (newSuburb == false)
			{
				String s = suburb.getText();
				if (s.length() == 0)
				{
					new MessageBox (this, "Error", "Cannot add a blank suburb");
					doIt = false;
				}
				else if (mySuburb.suburb.equals(s) == false &&
					 theServer.suburbExists (s))
				{
					new MessageBox (this, "Error", "This suburb already exists");
					doIt = false;
				}
			}

			if (doIt)
			{
				mySuburb.suburb = suburb.getText();
				mySuburb.postcode = postcode.getInt();
				mySuburb.ubd = ubdCode.getText();
				mySuburb.minOrder = minOrder.getDouble();
	
				if (newSuburb)
				{
					theServer.newSuburb (mySuburb);
				}
				else
				{
					theServer.updateSuburb (mySuburb);
				}
	
				CM.removeWindow ((GridFrame) this);
				dispose ();
				parentList.fillSuburbList();
			}
		}
		return (super.handleEvent (evt));
	}
}

class SuburbList extends List {

	public SuburbList ()
	{
		super (15, false);
	}

	public Dimension preferredSize ()
	{
		return new Dimension (251,251);
	}
}

class SuburbTable extends GridFrame {

	SuburbList suburbList;
	InputField searchThis;
	Button searchBtn;
	Button addBtn;
	Button delBtn;
	Button modBtn;
	Button cancelBtn;
	Server theServer;
	CreateMenu CM;

	public SuburbTable (CreateMenu c, Server s)
	{
		super ("Suburbs");
		CM = c;
		CM.addWindow ("SuburbTable", (GridFrame) this);
		theServer = s;
		initScreen ();
	}

	public void initScreen ()
	{
		searchThis = new InputField (30, InputField.CAPS);
		suburbList = new SuburbList ();
		suburbList.setBackground (Color.white);
		fillSuburbList ();

		searchBtn = new Button ("Search");
			searchBtn.setBackground (new Color (145,200,162));

		addBtn = new Button ("Add");
			addBtn.setBackground (new Color (145,200,162));

		modBtn = new Button ("Modify");
			modBtn.setBackground (new Color (145,196,255));

		delBtn = new Button ("Delete");
			delBtn.setBackground (new Color (255,128,128));

		cancelBtn = new Button ("Close");
			cancelBtn.setBackground (new Color (255,128,128));

		addElement (NORTHWEST, searchThis, 1,1,1,1);

		gridc.fill = 1;
		addElement (NORTHWEST, suburbList, 1,2,1,0);

		gridc.ipadx = gridc.ipady = 20;
		addElement (SOUTHEAST, searchBtn,   2,1,1,1);
		addElement (SOUTHEAST, addBtn,   2,2,1,1);
		addElement (SOUTHEAST, modBtn,   2,3,1,1);
		addElement (SOUTHEAST, delBtn,   2,4,1,1);
		addElement (SOUTHEAST, cancelBtn,2,5,1,1);

		pack();
		show();
		suburbList.requestFocus ();
	}

	void fillSuburbList ()
	{
		suburbList.clear ();
		String filter = searchThis.getText();
		for (int i =0; i < theServer.theSuburbs.size(); i++)
		{
			Suburb mySuburb = (Suburb) theServer.theSuburbs.elementAt (i);
			if (filter.length() == 0 || mySuburb.suburb.startsWith (filter))
			suburbList.addItem (mySuburb.suburb + " " + mySuburb.postcode + " " + mySuburb.ubd + " $" + Money.toString(mySuburb.minOrder));
		}
	}

	Suburb getSuburbAt (int i)
	{
		String filter = searchThis.getText();
		if (filter.length() == 0)
			return (Suburb) theServer.theSuburbs.elementAt (i);

		int iii = 0;
		for (int ii =0; ii < theServer.theSuburbs.size(); ii++)
		{
			Suburb mySuburb = (Suburb) theServer.theSuburbs.elementAt (ii);
			if (mySuburb.suburb.startsWith (filter))
			{
				if (iii == i)
					return mySuburb;
				iii++;
			}
		}
		return null;
	}

	public boolean handleEvent (Event evt)
	{
		if (evt.id == Event.KEY_PRESS && evt.key == '\n' && evt.target == suburbList)
		{
			evt.id = Event.ACTION_EVENT;
			evt.target = modBtn;
			evt.arg = "Modify";
			super.deliverEvent (evt);
			return (true);
		} else if (evt.id == Event.KEY_PRESS && evt.key == 0x1b)
		{
			CM.closeAllWindows ("SuburbEdit");
			CM.removeWindow ((GridFrame) this);
			dispose ();
		} else if ((evt.id == Event.WINDOW_DESTROY) ||
			   (evt.arg == "Close"))
		{
			CM.closeAllWindows ("SuburbEdit");
			CM.removeWindow ((GridFrame) this);
			dispose ();
		} else if (evt.arg == "Search" || evt.key == '\n')
		{
			fillSuburbList();
		} else if (evt.arg == "Add")
		{
			new SuburbEdit (CM, theServer, this);
		} else if (evt.arg == "Modify")
		{
			int i = suburbList.getSelectedIndex ();
			if (i != -1)
			{
			Suburb mySuburb = getSuburbAt (i);
			new SuburbEdit (CM, theServer, this, mySuburb);
			}

		} else if ((evt.arg == "Delete") || (evt.id == Event.KEY_PRESS && evt.key == 127))
		{
			int i = suburbList.getSelectedIndex ();
			if (i != -1)
			{
			Suburb mySuburb = getSuburbAt (i);
			theServer.deleteSuburb (mySuburb);
			fillSuburbList();
			}
		}

		return (super.handleEvent (evt));
	}
}

